	SUBROUTINE SENERGY(EXIT)
**********************************************************
*  Calculate total energy of a solid according to the    *
*  density functional theory.                            *
**********************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
	USE COM_SCFLOOP ,ONLY: EPSTOT                       !!! from readini
      USE COM_FORCES  ,ONLY: FPTN,FCOU,FEXC,FHLF          !!! from energy
      USE COM_UNIT    ,ONLY: IUN,IUR,IUT                  !!! from setup
C SET LOCAL VARIABLES
	IMPLICIT REAL*8 (A-H,O-Z)
	LOGICAL EXIT
	SAVE ETOT0
	DATA ETOT0/0.D0/
      CALL ABORTNOW()
C MEMORY ALLOCATIONS
      ALLOCATE(FPTN(3,NATOMMEM,4))
      ALLOCATE(FCOU(3,NATOMMEM,4))
      ALLOCATE(FEXC(3,NATOMMEM,4))
      ALLOCATE(FHLF(3,NATOMMEM,4))
C START CALCULATING ENERGIES
	CALL BANDENER(EBND)      !!! find band energy
	CALL COREENER(ECOR)      !!! find core energy
C POTENTIAL PART
	CALL POTFILES(12,'R','A')!!! read full potential
	CALL PTNLENER(EPTN)      !!! find potential energy
	CALL PTNFORCE(FPTN)      !!! find potential forces
	CALL HLFFORCE(FHLF)      !!! find Hellman-Feynman forces
	CALL POTFILES(12,'!','D')!!! deallocate full potential
C COULOMB PART
	CALL POTFILES(12,'!','A')!!! initialize Coulomb potential
	CALL COULENER(ECOU)      !!! find Coulomb energy
	CALL COUFORCE(FCOU)      !!! find Coulomb forces
	CALL POTFILES(12,'!','D')!!! deallocate Coulomb potential
C EXCHANGE-CORRELATION IN LDA/GGA
	CALL POTFILES(12,'!','A')!!! initialize exch.corr. potential
	CALL EXCHENER(EXCH)      !!! find exchange-correlation energy
	CALL EXCFORCE(FEXC)      !!! find exchange-correlation forces
	CALL POTFILES(12,'!','D')!!! deallocate exch.corr.potential
C EXCHANGE-CORRELATIONS BEYOND LDA/GGA
      CALL HUBRENER(EHUB)      !!! find Hubbard correction
C PRINT ENERGIES
	EKIN=EBND+ECOR-EPTN
	ETOT=EKIN+ECOU+EXCH+EHUB
	WRITE(IUN,1000)
	WRITE(IUN,1010)EBND
	WRITE(IUN,1020)ECOR
	WRITE(IUN,1030)EPTN
	WRITE(IUN,1040)EKIN
	WRITE(IUN,1050)ECOU
	WRITE(IUN,1060)EXCH
      WRITE(IUN,1065)EHUB
	WRITE(IUN,1070)ETOT
	IF(IUN.NE.IUT)
     &WRITE(IUT,1070)ETOT
	WRITE(IUN,1000)
	IF(ABS(ETOT-ETOT0).LE.EPSTOT)EXIT=.TRUE.
	IF(ABS(ETOT-ETOT0).GT.EPSTOT)EXIT=.FALSE.
	ETOT0=ETOT
1000  FORMAT(1X,'***************************************************')
1010  FORMAT(1X,'*       BAND ENERGY     =',G20.14,'     *')
1020  FORMAT(1X,'*       CORE ENERGY     =',G20.14,'     *')
1030  FORMAT(1X,'*       POTENTIAL ENERGY=',G20.14,'     *')
1040  FORMAT(1X,'*       KINETIC ENERGY  =',G20.14,'     *')
1050  FORMAT(1X,'*       COULOMB ENERGY  =',G20.14,'     *')
1060  FORMAT(1X,'*       EXCH-CORR ENERGY=',G20.14,'     *')
1065  FORMAT(1X,'*       HUBBARD ENERGY  =',G20.14,'     *')
1070  FORMAT(1X,'*       TOTAL ENERGY    =',G20.14,'     *')
	END




	SUBROUTINE BANDENER(EBND)
*************************************************************
*  Calculate one-electron contribution to the total energy  *
*************************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_PARAM  ,ONLY: NATOM,NSORT,NSPIN,NORBS,IS
     &                     ,ILST,ILSB,ILSV
      USE COM_NBANDS ,ONLY: NBNDF,NBNDE,NBNDT,NBNDC           !!! from readini
      USE COM_INPTTR ,ONLY: NDIV,NDIC,SH                      !!! from readini
      USE COM_KMESH  ,ONLY: NPNT,PNT,WGT,IPQ,IGQ,KCUT         !!! from makettr
      USE COM_ENERGY ,ONLY: EARR,WEIGH,WEIGD                  !!! from bands
      USE COM_KAPPAR ,ONLY: NPAN,NKAP,EKAP,EK                !!! from readini
      USE COM_INPPAR ,ONLY: ENY,DNY,TNY,DHLOG,MNY,LNY
     &                     ,NODES,ITS,NCLS
      USE COM_LABELS ,ONLY: TXTAT,TXTEL,TXTSP                !!! from readini
      USE COM_FERMI  ,ONLY: EF,ATOS,ADOS,PTOS,PFOS,PDOS      !!! from ttrints
      USE COM_UNIT   ,ONLY: IUN,IUR,IUT                      !!! from setup
C SET LOCAL VARIABLES
	IMPLICIT REAL*8 (A-H,O-Z)
	REAL*8, ALLOCATABLE :: TOS(:,:,:)
      CHARACTER TXTNY(0:3)*6,TL(0:3)*1,TN(9)*1
      DATA PI/3.141592653589793D0/,TL/'s','p','d','f'/,TN/'1','2'
     &    ,'3','4','5','6','7','8','9'/,TXTNY/'fix.E '
     &    ,'fix.D ','bind.E','center'/
C MEMORY ALLOCATIONS FOR MAIN CONTROL ARRAYS
	ALLOCATE(PTOS(0:LMAXTMEM,NSORTMEM,NKAPMEM,NPANMEM,
     &NSPINMEM,0:2))
	ALLOCATE(PFOS(0:LMAXTMEM,NSORTMEM,NKAPMEM,NPANMEM,
     &NSPINMEM,0:2))
	ALLOCATE(PDOS(0:LMAXTMEM,NSORTMEM,NKAPMEM,NPANMEM,
     &NSPINMEM,0:2))
	ALLOCATE(TOS(NPANMEM,0:1,NSPINMEM))
C START CODE
	DO IPAN=1,NPAN                          !!! over panels
	DO ISORT=1,NSORT                        !!! over sorts
	DO L=0,ILST(ISORT)                      !!! over l
C SET PARTIAL NUMBERS OF STATES
	DO ISPIN=1,NSPIN                        !!! over spins
	DO ICLS=1,NCLS(L,ISORT,IPAN)            !!! over l-classes
	PTOS(L,ISORT,ICLS,IPAN,ISPIN,0)=0.D0
	PFOS(L,ISORT,ICLS,IPAN,ISPIN,0)=0.D0
	PDOS(L,ISORT,ICLS,IPAN,ISPIN,0)=0.D0
	PTOS(L,ISORT,ICLS,IPAN,ISPIN,1)=0.D0
	PFOS(L,ISORT,ICLS,IPAN,ISPIN,1)=0.D0
	PDOS(L,ISORT,ICLS,IPAN,ISPIN,1)=0.D0
	PTOS(L,ISORT,ICLS,IPAN,ISPIN,2)=0.D0
	PFOS(L,ISORT,ICLS,IPAN,ISPIN,2)=0.D0
	PDOS(L,ISORT,ICLS,IPAN,ISPIN,2)=0.D0
	DO IKAP1=1,NKAP(IPAN)
	DO IKAP=1,NKAP(IPAN)
C TOTAL 
	PTOS(L,ISORT,ICLS,IPAN,ISPIN,0)=
     &PTOS(L,ISORT,ICLS,IPAN,ISPIN,0)+
     &ATOS(L,ISORT,IKAP1,IKAP,IPAN,ISPIN,0)
	PFOS(L,ISORT,ICLS,IPAN,ISPIN,0)=
     &PFOS(L,ISORT,ICLS,IPAN,ISPIN,0)+
     &ATOS(L,ISORT,IKAP1,IKAP,IPAN,ISPIN,1)
	PDOS(L,ISORT,ICLS,IPAN,ISPIN,0)=
     &PDOS(L,ISORT,ICLS,IPAN,ISPIN,0)+
     &ADOS(L,ISORT,IKAP1,IKAP,IPAN,ISPIN)
C DIAGONAL
	IF(LNY(L,ISORT,IKAP1,IPAN).EQ.ICLS.
     &AND.LNY(L,ISORT,IKAP,IPAN).EQ.ICLS)THEN
	PTOS(L,ISORT,ICLS,IPAN,ISPIN,1)=
     &PTOS(L,ISORT,ICLS,IPAN,ISPIN,1)+
     &ATOS(L,ISORT,IKAP1,IKAP,IPAN,ISPIN,0)
	PFOS(L,ISORT,ICLS,IPAN,ISPIN,1)=
     &PFOS(L,ISORT,ICLS,IPAN,ISPIN,1)+
     &ATOS(L,ISORT,IKAP1,IKAP,IPAN,ISPIN,1)
	PDOS(L,ISORT,ICLS,IPAN,ISPIN,1)=
     &PDOS(L,ISORT,ICLS,IPAN,ISPIN,1)+
     &ADOS(L,ISORT,IKAP1,IKAP,IPAN,ISPIN)
C CROSS
	ELSEIF((LNY(L,ISORT,IKAP1,IPAN).EQ.ICLS.
     &AND.LNY(L,ISORT,IKAP,IPAN).NE.ICLS).OR.
     &(LNY(L,ISORT,IKAP1,IPAN).NE.ICLS.
     &AND.LNY(L,ISORT,IKAP,IPAN).EQ.ICLS))THEN
	PTOS(L,ISORT,ICLS,IPAN,ISPIN,2)=
     &PTOS(L,ISORT,ICLS,IPAN,ISPIN,2)+
     &ATOS(L,ISORT,IKAP1,IKAP,IPAN,ISPIN,0)
	PFOS(L,ISORT,ICLS,IPAN,ISPIN,2)=
     &PFOS(L,ISORT,ICLS,IPAN,ISPIN,2)+
     &ATOS(L,ISORT,IKAP1,IKAP,IPAN,ISPIN,1)
	PDOS(L,ISORT,ICLS,IPAN,ISPIN,2)=
     &PDOS(L,ISORT,ICLS,IPAN,ISPIN,2)+
     &ADOS(L,ISORT,IKAP1,IKAP,IPAN,ISPIN)
	ENDIF
	ENDDO
	ENDDO
	ENDDO                                  !!! over l-classes
	ENDDO                                  !!! over spins
	ENDDO                                  !!! over l
	ENDDO                                  !!! over isort
	ENDDO                                  !!! over ipan
C PRINT PARTIAL NUMBERS OF STATES
	EBND=0.D0
	DOS =0.D0
	DO IPAN=1,NPAN                            !!! over panels
	DO ISPIN=1,NSPIN                          !!! over spins
	TOS(IPAN,0,ISPIN)=0.D0
	TOS(IPAN,1,ISPIN)=0.D0
	DO IPNT=1,NPNT(IPAN)                      !!! over k-points
	DO IBNDT=1,NBNDF(IPAN)                    !!! over bands 
	IBAND=ISPIN+NSPIN*(IBNDT-1)
	IF(IBNDT.GE.NBNDE(IPAN))THEN
	TOS(IPAN,0,ISPIN)=TOS(IPAN,0,ISPIN)+WEIGH(IBAND,IPNT)
	TOS(IPAN,1,ISPIN)=TOS(IPAN,1,ISPIN)+WEIGH(IBAND,IPNT)*
     &                  EARR(IBAND,IPNT,IPAN)
	DOS=DOS+WEIGD(IBAND,IPNT)
	ELSEIF(IBNDT.LT.NBNDE(IPAN))THEN
	TOS(IPAN,0,ISPIN)=TOS(IPAN,0,ISPIN)+WGT(IPNT,IPAN)/DBLE(NSPIN)
	TOS(IPAN,1,ISPIN)=TOS(IPAN,1,ISPIN)+WGT(IPNT,IPAN)/DBLE(NSPIN)*
     &EARR(IBAND,IPNT,IPAN)
	ENDIF
      ENDDO                                     !!! over valence bands
      ENDDO                                     !!! over k-points
      ENDDO                                     !!! over spins
C  AVERAGE IF SPIN-ORBIT COUPLING IS ON
	IF(NORBS.EQ.2)THEN
	DO I=0,1
	TBAR=(TOS(IPAN,I,1)+TOS(IPAN,I,NSPIN))/2.D0
	TOS(IPAN,I,1)=TBAR
	TOS(IPAN,I,NSPIN)=TBAR
	ENDDO
	ENDIF
      ENDDO                                     !!! over panels
C SET BAND ENERGY
	DO IPAN=1,NPAN                            !!! over panels
	EBND=EBND+(TOS(IPAN,1,1)+TOS(IPAN,1,NSPIN))/2.D0*DBLE(NSPIN)
	EBNP=(TOS(IPAN,1,1)+TOS(IPAN,1,NSPIN))/2.D0*DBLE(NSPIN)
	EAVR=(TOS(IPAN,1,1)+TOS(IPAN,1,NSPIN))/
     &     (TOS(IPAN,0,1)+TOS(IPAN,0,NSPIN))
	WRITE(IUN,1010)IPAN,EAVR,EBNP
	DO ISORT=1,NSORT                          !!! over sorts
      WRITE(IUN,1020)TXTEL(ISORT)
      WRITE(IUN,1030)(DREAL(PTOS(L,ISORT,1,IPAN,1,0)+
     &PTOS(L,ISORT,1,IPAN,NSPIN,0))/2.D0*NSPIN,
     &L=0,MIN(ILST(ISORT),3))
      IF(NSPIN.EQ.2)
     &WRITE(IUN,1040)(DREAL(PTOS(L,ISORT,1,IPAN,1,0)-
     &PTOS(L,ISORT,1,IPAN,NSPIN,0)),
     &L=0,MIN(ILST(ISORT),3))
      IF(IPAN.EQ.1.AND.NSPIN.EQ.1)
     &WRITE(IUN,1050)(DREAL(PDOS(L,ISORT,1,IPAN,1,0)),
     &L=0,MIN(ILST(ISORT),3))
      DO ISPIN=1,NSPIN
      IF(IPAN.EQ.1.AND.NSPIN.EQ.2)
     &WRITE(IUN,1060)TXTSP(ISPIN),(DREAL(
     &PTOS(L,ISORT,1,IPAN,ISPIN,0)),L=0,MIN(ILST(ISORT),3))
      ENDDO
      DO ISPIN=1,NSPIN
      IF(IPAN.EQ.1.AND.NSPIN.EQ.2)
     &WRITE(IUN,1070)TXTSP(ISPIN),(DREAL(
     &PDOS(L,ISORT,1,IPAN,ISPIN,0)),L=0,MIN(ILST(ISORT),3))
      ENDDO
      ENDDO                                 !!! over sorts
      ENDDO                                 !!! over panels
1010  FORMAT(1X,'Occupation numbers for the ',I2,'th panel; <E>=',
     &G14.7,'; Ebnd=',G14.7,'---->')
1020  FORMAT(1X,'Summed over tail energies partial states ',A10)
1030  FORMAT(1X,'spdf... TOS ='    ,20(1X,G11.4))
1040  FORMAT(1X,'spdf... MAG ='    ,20(1X,G11.4))
1050  FORMAT(1X,'spdf... DOS ='    ,20(1X,G11.4))
1060  FORMAT(1X,'spdf ',A2,'-TOS =',20(1X,G11.4))
1070  FORMAT(1X,'spdf ',A2,'-DOS =',20(1X,G11.4))
C PRINT CONTRIBUTIONS FROM DIFFERENT KAPPA'S
	IPAN=1
	WRITE(IUN,1110)
	DO ISORT=1,NSORT                          !!! over sorts
	DO ISPIN=1,NSPIN                          !!! over spins
	IF(NSPIN.EQ.1)WRITE(IUN,1120)TXTEL(ISORT)
	IF(NSPIN.EQ.2)WRITE(IUN,1130)TXTSP(ISPIN),TXTEL(ISORT)
	WRITE(IUN,1140)
	DO L=0,ILSB(ISORT)                        !!! over l
	DO ICLS=1,NCLS(L,ISORT,IPAN)              !!! over l-classes
	JKAP=0
	DO IKAP=1,NKAP(IPAN)
	IF(LNY(L,ISORT,IKAP,IPAN).EQ.ICLS)THEN
      MN=MNY(L,ISORT,IKAP,IPAN)
      NO=TNY(L,ISORT,IKAP,IPAN,1)+L+1
	JKAP=IKAP
	ENDIF
	ENDDO
	IF(JKAP.EQ.0)WRITE(IUT,*)'<BANDENER>:: ERROR IN BASIS DEFINITION'
	WRITE(IUN,1150)TN(NO)//TL(L),TXTNY(MN),
     &ENY(L,ISORT,JKAP,IPAN,ISPIN),
     &DREAL(PTOS(L,ISORT,ICLS,IPAN,ISPIN,1)),
     &DREAL(PTOS(L,ISORT,ICLS,IPAN,ISPIN,2))
	ENDDO                                     !!! over l-classes
	ENDDO                                     !!! over l
	ENDDO                                     !!! over spins
	ENDDO                                     !!! over sorts
C MEMORY DEALLOCATIONS
	DEALLOCATE(TOS)
1110	FORMAT(1X,'Different contributions for the orbitals ',
     &'with many kappas --->')
1120	FORMAT(1X,'Diagonal/cross partial occupations ',A10)
1130	FORMAT(1X,'Diagonal/cross spin-',A2,' occupations ',A10)
1140	FORMAT(1X,'State  Eny',8X,'Value(Ry)',7X,'DTOS',13X,'CTOS :')
1150	FORMAT(2X,A2,3X,A6,3X,G12.5,3X,G14.7,1X,G14.7)
	END





	SUBROUTINE COREENER(ECOR)
*************************************************************
*  Calculate one-electron contribution to the total energy  *
*************************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_PARAM   ,ONLY: NATOM,NSORT,NSPIN,NORBS
     &                      ,IS,ILSV,NHSORT                   !!! from readini
      USE COM_CORELEC ,ONLY: NLEVT,LEVC,ELEV,ZLEV             !!! from readscf
      USE COM_POTMTS  ,ONLY: VMT0                             !!! from vzero
      USE COM_UNIT    ,ONLY: IUN,IUR,IUT                      !!! from setup
C SET LOCAL VARIABLES
	IMPLICIT REAL*8 (A-H,O-Z)
	DATA PI/3.141592653589793D0/
C  CALCULATE ONE-ELECTRON CORE CONTRIBUTION
	ECOR=0.D0
	DO ISORT=1,NSORT
	DO ILEV=1,NLEVT(ISORT)
	IF(LEVC(ILEV,ISORT).EQ.1)
     &ECOR=ECOR+ELEV(ILEV,ISORT)*NHSORT(ISORT)*ZLEV(ILEV,ISORT)
	ENDDO
	ENDDO
	END


	SUBROUTINE PTNLENER(EPTN)
*****************************************************************
*  Calculate potential energy contribution to the total energy  *
*****************************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_CONTROL ,ONLY: LMTO,IASA,IEXC                   !!! from readini
      USE COM_PARAM   ,ONLY: NATOM,NSORT,NSPIN,NORBS
     &                      ,IS,ILSV,NHSORT                   !!! from readini
      USE COM_LATPAR  ,ONLY: PAR,OMEGAC                       !!! from readini
      USE COM_SPHERE  ,ONLY: ZNUC,ZCOR,SMT,SWZ                !!! from readini
	USE COM_SYMMLM  ,ONLY: NSYM,LMSYM,SYM                   !!! from symlmm
	USE COM_DENDAT  ,ONLY: RHO,RHOCOR,RHOPS,RHOPS1          !!! from potfull
	USE COM_POTDAT  ,ONLY: POT,POTSPH,POTPS,POTPS1          !!! from rhofull
	USE COM_POTMTS  ,ONLY: VMT0                             !!! from potzero
      USE COM_RADMESH ,ONLY: NRAD,H,R,DR,WEIGHT,RL            !!! from radmesh
      USE COM_PLWMESH ,ONLY: NPLW,GPLW                        !!! from plwmesh
      USE COM_UNIT    ,ONLY: IUN,IUR,IUT                      !!! from setup
C SET LOCAL VARIABLES
	IMPLICIT REAL*8 (A-H,O-Z)
	REAL*8, ALLOCATABLE :: WORK3(:)
	REAL*8, ALLOCATABLE :: WORK4(:)
	COMPLEX*16, ALLOCATABLE :: WORK1(:)
	COMPLEX*16, ALLOCATABLE :: WORK2(:)
	DATA PI/3.141592653589793D0/
	SQPI4=2.D0*SQRT(PI)
C MEMORY ALLOCATIONS
	ALLOCATE(WORK1(0:NRADMEM))
	ALLOCATE(WORK2(0:NRADMEM))
	ALLOCATE(WORK3(0:NRADMEM))
	ALLOCATE(WORK4(0:NRADMEM))
C CALCULATE MUFFIN-TIN CONTRIBUTION TO THE POTENTIAL ENERGY
	EPTNMT=0.D0
	DO IATOM=1,NATOM                                    !!! over atoms
	ISORT=IS(IATOM)
	DO IORBS=1,NORBS                                    !!! over iorbs
	DO IUPDN=1,NSPIN                                    !!! over spins
	DO L=0,ILSV(ISORT)                                  !!! over l
	DO M=-L,L                                           !!! over m
	LM=L*(L+1)+M+1
	IF(SYM(LM,IATOM))THEN
	ISYM=LMSYM(LM,IATOM)
	DO IRAD=0,NRAD(ISORT)                               !!! over irad
	WORK1(IRAD)=DCONJG(RHO(IRAD,ISYM,IATOM,IUPDN,IORBS))*
     &POT(IRAD,ISYM,IATOM,IUPDN,IORBS)*DR(IRAD,ISORT)*R(IRAD,ISORT)**2
      ENDDO                                               !!! over irad
	CALL CQD(H(ISORT),WORK1,WORK2,NRAD(ISORT)+1)
	EPTNMT=EPTNMT+WORK2(NRAD(ISORT))
	ENDIF
      ENDDO                                               !!! over m
      ENDDO                                               !!! over l
      ENDDO                                               !!! over ispin
      ENDDO                                               !!! over iorbs
      ENDDO                                               !!! over iatom
C  CALCULATE PSEUDOPOTENTIAL CONTRIBUTION FROM INT.REGION
	EPTNINT=0.D0
	DO IORBS=1,NORBS                                    !!! over iorbs
	DO IUPDN=1,NSPIN                                    !!! over spins
	DO IPLW=0,NPLW                                      !!! over plane waves
	EPTNINT=EPTNINT+POTPS1(IPLW,IUPDN,IORBS)*
     &DCONJG(RHOPS(IPLW,IUPDN,IORBS))
      ENDDO                                               !!! over plane waves
      ENDDO                                               !!! over spins
      ENDDO                                               !!! over iorbs
	EPTN=EPTNMT+EPTNINT
	WRITE(IUN,1010)EPTNMT,EPTNINT
C MEMORY DEALLCATIONS
	DEALLOCATE(WORK1)
	DEALLOCATE(WORK2)
	DEALLOCATE(WORK3)
	DEALLOCATE(WORK4)
1010  FORMAT(1X,'E{ptn} : MT,INT-parts are',2(1X,G14.7))
	END




	SUBROUTINE COULENER(ECOU)
**************************************************************
*  Calculate electrostatic contribution to the total energy  *
**************************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_CONTROL ,ONLY: LMTO,IASA,IEXC                   !!! from readini
      USE COM_PARAM   ,ONLY: NATOM,NSORT,NSPIN,NORBS
     &                      ,IS,ILSV,NHSORT                   !!! from readini
      USE COM_LATPAR  ,ONLY: PAR,OMEGAC                       !!! from readini
      USE COM_SPHERE  ,ONLY: ZNUC,ZCOR,SMT,SWZ                !!! from readini
      USE COM_SYMMLM  ,ONLY: NSYM,LMSYM,SYM                   !!! from symlmm
      USE COM_DENDAT  ,ONLY: RHO,RHOCOR,RHOPS,RHOPS1          !!! from potfull
      USE COM_POTDAT  ,ONLY: POT,POTSPH,POTPS,POTPS1          !!! from rhofull
      USE COM_POTMTS  ,ONLY: VMT0                             !!! from potzero
      USE COM_RADMESH ,ONLY: NRAD,H,R,DR,WEIGHT,RL            !!! from radmesh
      USE COM_PLWMESH ,ONLY: NPLW,GPLW                        !!! from plwmesh
      USE COM_UNIT    ,ONLY: IUN,IUR,IUT                      !!! from setup
C SET LOCAL VARIABLES
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8, ALLOCATABLE :: WORK3(:)
      REAL*8, ALLOCATABLE :: WORK4(:)
      COMPLEX*16, ALLOCATABLE :: WORK1(:)
      COMPLEX*16, ALLOCATABLE :: WORK2(:)
      DATA PI/3.141592653589793D0/
	SQPI4=2.D0*SQRT(PI)
C MEMORY ALLOCATIONS
      ALLOCATE(WORK1(0:NRADMEM))
      ALLOCATE(WORK2(0:NRADMEM))
      ALLOCATE(WORK3(0:NRADMEM))
      ALLOCATE(WORK4(0:NRADMEM))
C CALCULATE MULTIPOLE FIELD
      CALL MULTRHO           !!! calculate moments of (pseudo)density
      CALL MULTFTR           !!! multipoles as lattice sums
      CALL MULTASA           !!! multipoles as lattice sums in ASA
C CALCULATE COULOMB POTENTIAL WITHOUT NUCLEAR PART
	CALL VCOUL             !!! calculate Coulomb potential whithin MTs
	CALL VCFTR             !!! calculate Coulomb potential in the int.reg.
	CALL VPINT             !!! set up convolutions
C CALCULATE MT-CONTRIBUTION TO THE COULOMB ENERGY
	ECOUMT=0.D0
	DO IATOM=1,NATOM                                     !!! over atoms
	ISORT=IS(IATOM)
	DO L=0,ILSV(ISORT)                                   !!! over l
	DO M=-L,L                                            !!! over m
	LM=L*(L+1)+M+1
	IF(SYM(LM,IATOM))THEN
	ISYM=LMSYM(LM,IATOM)
	IF(L.EQ.0)THEN
	DO IRAD=0,NRAD(ISORT)                                !!! over irad
	TR1=0.5D0*DREAL(POT(IRAD,1,IATOM,1,1)+POT(IRAD,1,IATOM,NSPIN,1))/
     &2.D0*R(IRAD,ISORT)**2*DR(IRAD,ISORT)
	TR2=DREAL(RHO(IRAD,1,IATOM,1,1)+RHO(IRAD,1,IATOM,NSPIN,1))/
     &2.D0*DFLOAT(NSPIN)
	WORK3(IRAD)=TR1*TR2
      ENDDO                                                !!! over irad
	CALL DQD(H(ISORT),WORK3,WORK4,NRAD(ISORT)+1)
	ECOUMT=ECOUMT+WORK4(NRAD(ISORT))
	DO IRAD=0,NRAD(ISORT)                                !!! over irad
	TR1=-ZNUC(ISORT)*SQPI4*R(IRAD,ISORT)*DR(IRAD,ISORT)
	TR2=DREAL(RHO(IRAD,1,IATOM,1,1)+RHO(IRAD,1,IATOM,NSPIN,1))/
     &2.D0*DFLOAT(NSPIN)
	WORK3(IRAD)=TR1*TR2
      ENDDO                                                !!! over irad
	CALL DQD(H(ISORT),WORK3,WORK4,NRAD(ISORT)+1)
	ECOUMT=ECOUMT+WORK4(NRAD(ISORT))-0.5D0*ZNUC(ISORT)*(
     &POT(0,1,IATOM,1,1)+POT(0,1,IATOM,NSPIN,1))/2.D0/SQPI4
	ELSE
	DO IRAD=0,NRAD(ISORT)                                !!! over irad
	WORK1(IRAD)=0.5D0*DCONJG(RHO(IRAD,ISYM,IATOM,1,1)+
     &RHO(IRAD,ISYM,IATOM,NSPIN,1))/2.D0*DFLOAT(NSPIN)*(
     &POT(IRAD,ISYM,IATOM,1,1)+POT(IRAD,ISYM,IATOM,NSPIN,1))/
     &2.D0*R(IRAD,ISORT)**2*DR(IRAD,ISORT)
      ENDDO                                                !!! over irad
	CALL CQD(H(ISORT),WORK1,WORK2,NRAD(ISORT)+1)
	ECOUMT=ECOUMT+WORK2(NRAD(ISORT))
	ENDIF
	ENDIF
      ENDDO                                                !!! over m
      ENDDO                                                !!! over l
      ENDDO                                                !!! over iatom
C  CALCULATE PSEUDOPOTENTIAL CONTRIBUTION FROM INT.REGION 
	ECOUINT=0.D0
	ECOUZER=0.D0
	DO IPLW=0,NPLW                                       !!! over plane waves
	ECOUINT=ECOUINT+(POTPS1(IPLW,1,1)+POTPS1(IPLW,NSPIN,1))/2.D0*
     &DCONJG(RHOPS (IPLW,1,1)+RHOPS (IPLW,NSPIN,1))/2.D0*DBLE(NSPIN)
	ECOUZER=ECOUZER+(POTPS1(IPLW,1,1)+POTPS1(IPLW,NSPIN,1))/2.D0*
     &DCONJG(RHOPS1(IPLW,1,1)+RHOPS1(IPLW,NSPIN,1))/2.D0*DBLE(NSPIN)
      ENDDO                                                !!! over spins
	ECOUINT=0.5D0*ECOUINT
	ECOUZER=0.5D0*ECOUZER
	ECOU=ECOUMT+ECOUINT
	WRITE(IUN,1010)ECOUMT,ECOUINT,ECOUZER
1010  FORMAT(1X,'E{cou} : M,I,Z0-parts are',3(1X,G14.7))
	DEALLOCATE(WORK1)
	DEALLOCATE(WORK2)
	DEALLOCATE(WORK3)
	DEALLOCATE(WORK4)
	END


	SUBROUTINE EXCHENER(EXCH)
*********************************************************
*  Calcculate exchange-correlation contribution to the  *
*  total energy.                                        *
*********************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_CONTROL ,ONLY: LMTO,IASA,IEXC                   !!! from readini
      USE COM_PARAM   ,ONLY: NATOM,NSORT,NSPIN,NORBS
     &                      ,IS,ILSV,NHSORT                   !!! from readini
      USE COM_LATPAR  ,ONLY: PAR,OMEGAC                       !!! from readini
      USE COM_MAGDAT  ,ONLY: AXMAG,BEXT,ISPL,SPLIT            !!! from readini
      USE COM_SPHERE  ,ONLY: ZNUC,ZCOR,SMT,SWZ                !!! from readini
      USE COM_SYMMLM  ,ONLY: NSYM,LMSYM,SYM                   !!! from symlmm
      USE COM_DENDAT  ,ONLY: RHO,RHOCOR,RHOPS,RHOPS1          !!! from potfull
      USE COM_POTDAT  ,ONLY: POT,POTSPH,POTPS,POTPS1          !!! from rhofull
      USE COM_POTMTS  ,ONLY: VMT0                             !!! from potzero
      USE COM_RADMESH ,ONLY: NRAD,H,R,DR,WEIGHT,RL            !!! from radmesh
      USE COM_PLWMESH ,ONLY: NPLW,GPLW                        !!! from plwmesh
      USE COM_UNIT    ,ONLY: IUN,IUR,IUT                      !!! from setup
C SET LOCAL VARIABLES
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8, ALLOCATABLE :: WORK3(:)
      REAL*8, ALLOCATABLE :: WORK4(:)
      COMPLEX*16, ALLOCATABLE :: WORK1(:)
      COMPLEX*16, ALLOCATABLE :: WORK2(:)
	COMPLEX*16 HEXT(-1:1),AMAG(-1:1)
      DATA PI/3.141592653589793D0/
	SQPI4=2.D0*SQRT(PI)
C MEMORY ALLOCATIONS
      ALLOCATE(WORK1(0:NRADMEM))
      ALLOCATE(WORK2(0:NRADMEM))
      ALLOCATE(WORK3(0:NRADMEM))
      ALLOCATE(WORK4(0:NRADMEM))
C APPLY EXTERNAL MAGNETIC FIELD ALONG LOCAL AXIS
C THIS WILL JUST SPLIT SPIN-UP AND SPIN-DN POTENTIAL 
C FOR AXIS ALONG (0,0,1)
      DO IATOM=1,NATOM                    !!! over atoms
      ISORT=IS(IATOM)
	AMAG(+0)=  AXMAG(3,ISORT)
	AMAG(-1)=+(AXMAG(1,ISORT)+(0.D0,1.D0)*AXMAG(2,ISORT))/SQRT(2.D0)
	AMAG(+1)=-(AXMAG(1,ISORT)-(0.D0,1.D0)*AXMAG(2,ISORT))/SQRT(2.D0)
	HEXT(+0)=AMAG(+0)*(SPLIT(ISORT)+BEXT(ISORT))
	HEXT(-1)=AMAG(-1)*(SPLIT(ISORT)+BEXT(ISORT))
	HEXT(+1)=AMAG(+1)*(SPLIT(ISORT)+BEXT(ISORT))
      DO IRAD=0,NRAD(ISORT)
      POT(IRAD,1,IATOM,1,1)=POT(IRAD,1,IATOM,1,1)+
     &HEXT(+0)*SQPI4
      POT(IRAD,1,IATOM,NSPIN,1)=POT(IRAD,1,IATOM,NSPIN,1)-
     &HEXT(+0)*SQPI4
	IF(NORBS.EQ.2)THEN
      POT(IRAD,1,IATOM,1,NORBS)=POT(IRAD,1,IATOM,1,NORBS)+
     &HEXT(-1)*SQPI4
      POT(IRAD,1,IATOM,NSPIN,NORBS)=POT(IRAD,1,IATOM,NSPIN,NORBS)+
     &HEXT(+1)*SQPI4
	ENDIF
      ENDDO
      ENDDO                               !!! over atoms
C  Switch on in the interstitials?
	ANX=AXMAG(1,1)
	ANY=AXMAG(2,1)
	ANZ=AXMAG(3,1)
	SPL=0.D0
	BXT=0.D0
	LA=1
	DO ISORT=1,NSORT
	IF(ABS(ANX-AXMAG(1,ISORT)).GE.1.D-07.OR.
     &   ABS(ANY-AXMAG(2,ISORT)).GE.1.D-07.OR.
     &   ABS(ANZ-AXMAG(3,ISORT)).GE.1.D-07)LA=0
	SPL=SPL+SPLIT(ISORT)*DBLE(NHSORT(ISORT))/DBLE(NATOM)
	BXT=BXT+BEXT (ISORT)*DBLE(NHSORT(ISORT))/DBLE(NATOM)
	ENDDO
	IF(LA.EQ.1)THEN ! switch on
	AMAG(+0)=  ANZ
	AMAG(-1)=+(ANX+(0.D0,1.D0)*ANY)/SQRT(2.D0)
	AMAG(+1)=-(ANX-(0.D0,1.D0)*ANY)/SQRT(2.D0)
	HEXT(+0)=AMAG(+0)*(SPL+BXT)
	HEXT(-1)=AMAG(-1)*(SPL+BXT)
	HEXT(+1)=AMAG(+1)*(SPL+BXT)
      IF(NPLW.NE.-1)POTPS(0,1,1)=POTPS(0,1,1)+HEXT(+0)
      IF(NPLW.NE.-1)POTPS(0,NSPIN,1)=POTPS(0,NSPIN,1)-HEXT(+0)
	IF(NORBS.EQ.2)THEN
      IF(NPLW.NE.-1)POTPS(0,1,NORBS)=POTPS(0,1,NORBS)+HEXT(-1)
      IF(NPLW.NE.-1)POTPS(0,NSPIN,NORBS)=POTPS(0,NSPIN,NORBS)+HEXT(+1)
	ENDIF
	ENDIF
C CALCULATE DENSITY OF THE EXCHANGE-CORRELATION ENERGY
	CALL VEXCH(2)           !!! exchange-correlation within MTs
	CALL VXFTR(2)           !!! pseudopart by Fourier transform
	CALL VPINT              !!! set up convolutions
C CALCULATE MT-CONTRIBUTION TO THE EXCHANGE-CORRELATION ENERGY
	EXCHMT=0.D0
	DO IORBS=1,NORBS                                    !!! over iorbs
	DO IUPDN=1,NSPIN                                    !!! over spins
	DO IATOM=1,NATOM                                    !!! over atoms
	ISORT=IS(IATOM)
	DO L=0,ILSV(ISORT)                                  !!! over l
	DO M=-L,L                                           !!! over m
	LM=L*(L+1)+M+1
	IF(SYM(LM,IATOM))THEN
	ISYM=LMSYM(LM,IATOM)
	DO IRAD=0,NRAD(ISORT)                               !!! over irad
	WORK1(IRAD)=DCONJG(RHO(IRAD,ISYM,IATOM,IUPDN,IORBS))*
     &POT(IRAD,ISYM,IATOM,IUPDN,IORBS)*R(IRAD,ISORT)**2*DR(IRAD,ISORT)
      ENDDO                                               !!! over irad
	CALL CQD(H(ISORT),WORK1,WORK2,NRAD(ISORT)+1)
	EXCHMT=EXCHMT+WORK2(NRAD(ISORT))
	ENDIF
      ENDDO                                               !!! over m
      ENDDO                                               !!! over l
      ENDDO                                               !!! over iatom
      ENDDO                                               !!! over ispin
      ENDDO                                               !!! over iorbs
C CALCULATE PSEUDOPOTENTIAL CONTRIBUTION FROM INT.REGION 
	EXCHINT=0.D0
	DO IORBS=1,NORBS                                    !!! over iorbs
	DO IUPDN=1,NSPIN                                    !!! over spins
	DO IPLW=0,NPLW                                      !!! over plane waves
	EXCHINT=EXCHINT+POTPS1(IPLW,IUPDN,IORBS)*
     &DCONJG(RHOPS(IPLW,IUPDN,IORBS))
      ENDDO                                               !!! over plane waves
      ENDDO                                               !!! over spins
      ENDDO                                               !!! over iorbs
	EXCH=EXCHMT+EXCHINT
	WRITE(IUN,1010)EXCHMT,EXCHINT
1010  FORMAT(1X,'E{xc } : MT,INT-parts are',2(1X,G14.7))
	DEALLOCATE(WORK1)
	DEALLOCATE(WORK2)
	DEALLOCATE(WORK3)
	DEALLOCATE(WORK4)
	END
