      SUBROUTINE READSCF
************************************************************
*  Reads charge density destribution in cell               *
************************************************************
C SET ARRAY SIZE DATA
	USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                             !!! from setup
      USE COM_CONTROL ,ONLY: LMTO,IASA,LRWF                      !!! from readini
      USE COM_PARAM   ,ONLY: NATOM,NSORT,NSPIN,NORBS,IS          !!! from readini
      USE COM_MAGDAT  ,ONLY: ISPL,SPLIT                          !!! from readini
	USE COM_FERMI   ,ONLY: EF                                  !!! from readscf
      USE COM_LMTINP  ,ONLY: PROJECT,INPINFO,SCSFILE,SCSMADE,SCFMADE!from readini
	USE COM_EXTENS  ,ONLY: EXTNSCS,LENEXTN                     !!! from setup
      USE COM_UNIT    ,ONLY: IUN,IUR,IUT                         !!! from setup
C SET LOCAL VARIABLES
      IMPLICIT REAL*8 (A-H,O-Z)
	INTEGER, ALLOCATABLE :: IS0(:)             ! old atom-to-sort pointer
	REAL*8, ALLOCATABLE :: ATMAP(:,:)          ! maps atoms in old and new structures
	REAL*8, ALLOCATABLE :: SOMAP(:,:)          ! maps sorts in old and new structures
	REAL*8, ALLOCATABLE :: SPMAP(:,:)          ! maps spins in old and new structures
	REAL*8, ALLOCATABLE :: SVMAP(:,:)          ! maps spins in old and new structures
	COMPLEX*16, ALLOCATABLE :: SRMAP(:,:,:,:,:)! maps magnetization densities
	LOGICAL SAME,EXIST,PRESENT
	CHARACTER INF*130,TEXT*64
	IF(IASA.EQ.0)RETURN
C CHECK INPINFO
      INF=INPINFO
      CALL SUBSTR_IN_STR(PRESENT,INF,130,EXTNSCS,LENEXTN)
      IF(.NOT.PRESENT)CALL READSCF_MESG00
C SET FILE NAME
      CALL FIND_LBLANK(PROJECT,130,NBP)
      CALL FIND_LBLANK(EXTNSCS,60,NBE)
      SCSFILE=PROJECT(:NBP)//'.'//EXTNSCS(:NBE)
C CHECK FILE EXISTENCE
      INQUIRE(FILE=SCSFILE,EXIST=EXIST)
      IF(.NOT.EXIST)CALL READSCF_MESG01
      WRITE(IUN,1010)
1010  FORMAT(1X,'Start reading SCFFILE:   '/)
C INITIALIZE DENSITY ARRAYS
	CALL RHOFILES(4,'!','A')
C READ MAIN CONTROL DATA
	OPEN(UNIT=1,FILE=SCSFILE,STATUS='OLD',FORM='FORMATTED')
	READ(1,FMT='(A)')TEXT
	IF(TEXT(2:2).NE.'<')BACKSPACE(1)
	READ(1,*)NATOM0,NSORT0,NSPIN0,NORBS0
	ALLOCATE(IS0(NATOM0))
	READ(1,*)(IS0(IATOM0),IATOM0=1,NATOM0)
C Allocate mapping arrays
	ALLOCATE(ATMAP(NATOM,NATOM0))
	ALLOCATE(SOMAP(NSORT,NSORT0))
	ALLOCATE(SPMAP(NSPIN,NSPIN0))
	ALLOCATE(SVMAP(NSPIN,NSPIN0))
	ALLOCATE(SRMAP(NSPIN,NORBS,NSPIN0,NORBS0,0:NSORT))
C TRY TO GET CORRESPONDENCE BETWEEN TWO STRUCTURES
	CALL READSCF_MAPS(NATOM0,NSORT0,NSPIN0,NORBS0,IS0,
     &     ATMAP,SOMAP,SPMAP,SVMAP,SRMAP,SAME)
C READ CHARGE DENSITY WITHIN SPHERES
	CALL READSCF_RHOS(NATOM0,NSORT0,NSPIN0,NORBS0,IS0,
     &     ATMAP,SOMAP,SPMAP,SVMAP,SRMAP,SAME)
C READ INTERSTITIAL DENSITY
	CALL READSCF_RHOI(NATOM0,NSORT0,NSPIN0,NORBS0,IS0,
     &     ATMAP,SOMAP,SPMAP,SVMAP,SRMAP,SAME)
C READ RESTART INFORMATION
	CALL READSCF_REST(NATOM0,NSORT0,NSPIN0,NORBS0,IS0,
     &     ATMAP,SOMAP,SPMAP,SVMAP,SRMAP,SAME)
C READ FERMI ENERGY, SCF KEY
	READ(1,*)EF,SCFMADE
	CLOSE(1)
C CONTROL SPIN SPLITTING
      DO ISORT=1,NSORT
      IF(SCFMADE.AND.NSPIN.EQ.NSPIN0.AND.ISPL(ISORT).EQ.0)
     &SPLIT(ISORT)=0.D0
      ENDDO
C CHECK POTENTIAL KEY
      IF(.NOT.SCFMADE.AND.LRWF.EQ.0)THEN
      WRITE(IUT,*)'Warning from <READSCF>:'
      WRITE(IUT,*)'Since density read from SCFFILE is NOT'
	WRITE(IUT,*)'self-consistent, no frozing'
      WRITE(IUT,*)'potential key LRWF=0 avaiable. LRWF is set to 2,'
      WRITE(IUT,*)'i.e. to fully adjusted potential.'
      LRWF=2
      ENDIF
C READING IS DONE, SAVE DENSITY
	CALL RHOFILES(4,'W','D')
C MEMORY DEALLOCATIONS
	DEALLOCATE(IS0)
	DEALLOCATE(ATMAP)
	DEALLOCATE(SOMAP)
	DEALLOCATE(SPMAP)
	DEALLOCATE(SVMAP)
	CONTAINS
C ERROR HANDLINGS
      SUBROUTINE READSCF_MESG00
      WRITE(IUT,*)'Terminated in <READSCF>:'
      WRITE(IUT,*)'SCSFILE identificator << '
     &//EXTNSCS(1:LENEXTN)//' >>'
      WRITE(IUT,*)'is not found in INPINFO string.'
      CALL FULLSTOP(' ')
      END SUBROUTINE READSCF_MESG00
      SUBROUTINE READSCF_MESG01
      WRITE(IUT,*)'Terminated in <READSCF>:'
      WRITE(IUT,*)'Input data file SCFFILE ',SCSFILE
      WRITE(IUT,*)'cannot be found. Check for its existence'
      WRITE(IUT,*)'and/or correct path.'
      CALL FULLSTOP(' ')
      END SUBROUTINE READSCF_MESG01
	END SUBROUTINE READSCF


      SUBROUTINE READSCF_MAPS(NATOM0,NSORT0,NSPIN0,NORBS0,IS0,
     &           ATMAP,SOMAP,SPMAP,SVMAP,SRMAP,SAME)
***********************************************************************
* Tries to find correspondence between old and new structures.        *
***********************************************************************
C SET ARRAY SIZE DATA
	USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_PARAM,   ONLY : NATOM,NSORT,NSPIN,NORBS,IS          !!! from readini
      USE COM_SPHERE,  ONLY : ZNUC,SMT                            !!! from readini
      USE COM_LATPAR,  ONLY : PAR,ORTH,OMEGAC,STRAIN              !!! from readini
      USE COM_LATTICE, ONLY : RBAS,GBAS,TAU,TAO                   !!! from readstr
      USE COM_LABELS,  ONLY : TXTEL,TXTAT                         !!! from readini
      USE COM_MAGDAT,  ONLY : AXMAG                               !!! from ttrint1
      USE COM_LMTINP,  ONLY : SMTMADE,SCSMADE                     !!! from readini
      USE COM_UNIT,    ONLY : IUN,IUR,IUT                         !!! from readini
C SET LOCAL VARIABLES
      IMPLICIT REAL*8 (A-H,O-Z)
	REAL*8,  ALLOCATABLE :: TAU0(:,:)      ! old basis
	REAL*8,  ALLOCATABLE :: TAO0(:,:)      ! old basis
	REAL*8,  ALLOCATABLE :: ZNUC0(:)       ! old nuclear charges
	REAL*8,  ALLOCATABLE :: SMT0(:)        ! old MT-spheres
	REAL*8,  ALLOCATABLE :: SMT1(:)        ! new MT-spheres
	REAL*8,  ALLOCATABLE :: AXMAG0(:,:)    ! old magnetization axis
	REAL*8,  ALLOCATABLE :: TAO1E(:,:)
	REAL*8,  ALLOCATABLE :: TAO0E(:,:)
	INTEGER IS0(NATOM0)
	REAL*8 ATMAP(NATOM,NATOM0),SOMAP(NSORT,NSORT0)
     &      ,SPMAP(NSPIN,NSPIN0),SVMAP(NSPIN,NSPIN0)
     &      ,STRAIN0(3,3,2),ORTH0(2:3),TR1(3),TR0(3)
     &      ,RBAS1E(3,3),GBAS1E(3,3),RBAS0E(3,3),GBAS0E(3,3)
     &      ,RBAS0(3,3),GBAS0(3,3)
      COMPLEX*16 SRMAP(NSPIN,NORBS,NSPIN0,NORBS0,0:NSORT),TR(3,-1:1)
     &      ,TD(-1:1,3),GD(4,-1:2),GR(-1:2,4),TRN(3,3),TRO(3,3)
     &      ,TRC(3,3),TRM(-1:2,-1:2),TUD(4,4),ACOLD(4),AMOLD(-1:2)
     &      ,ROOLD(2,2),ACNEW(4),AMNEW(-1:2),RONEW(2,2)
	LOGICAL SAME
      DATA PI/3.141592653589793D0/
C READ CONTROL DATA
	ALLOCATE(TAU0(3,NATOM0))
	ALLOCATE(TAO0(3,NATOM0))
	ALLOCATE(ZNUC0(NSORT0))
	ALLOCATE(SMT0 (NSORT0))
	ALLOCATE(SMT1 (NSORT))
	ALLOCATE(AXMAG0(3,NSORT))
      READ(1,*)RBAS0,GBAS0,PARR0,ORTH0,STRAIN0,
     &((TAO0(I,IATOM0),I=1,3),IATOM0=1,NATOM0),
     &((TAU0(I,IATOM0),I=1,3),IATOM0=1,NATOM0),
     &(ZNUC0(ISORT0),ISORT0=1,NSORT0),
     &(SMT0(ISORT0),ISORT0=1,NSORT0),
     &((AXMAG0(I,ISORT0),I=1,3),ISORT0=1,NSORT0)
C SET LATTICE WITHOUT STRAINS AND DISPLACEMENTS
	ALLOCATE(TAO0E(3,NATOM0))
	ALLOCATE(TAO1E(3,NATOM))
C Translations
	DO IVEC=1,3
	DO I=1,3
	RBAS1E(I,IVEC)=0.D0	
	RBAS0E(I,IVEC)=0.D0	
	GBAS1E(I,IVEC)=0.D0	
	GBAS0E(I,IVEC)=0.D0	
	DO J=1,3
      RBAS1E(I,IVEC)=RBAS1E(I,IVEC)+STRAIN (I,J,2)*RBAS (J,IVEC)
      RBAS0E(I,IVEC)=RBAS0E(I,IVEC)+STRAIN0(I,J,2)*RBAS0(J,IVEC)
      GBAS1E(I,IVEC)=GBAS1E(I,IVEC)+GBAS (J,IVEC)*STRAIN (J,I,1)
      GBAS0E(I,IVEC)=GBAS0E(I,IVEC)+GBAS0(J,IVEC)*STRAIN0(J,I,1)
	ENDDO
	ENDDO
	ENDDO
C Basis vectors
	DO IATOM=1,NATOM
	DO I=1,3
	TAO1E(I,IATOM)=0.D0	
	DO J=1,3
      TAO1E(I,IATOM)=TAO1E(I,IATOM)+STRAIN(I,J,2)*TAO(J,IATOM)
	ENDDO
	ENDDO
	ENDDO
	DO IATOM0=1,NATOM0
	DO I=1,3
	TAO0E(I,IATOM0)=0.D0	
	DO J=1,3
      TAO0E(I,IATOM0)=TAO0E(I,IATOM0)+STRAIN0(I,J,2)*TAO0(J,IATOM0)
	ENDDO
	ENDDO
	ENDDO
C SET SPIN MAPPING
	IF(NSPIN.EQ.NSPIN0)THEN
	DO ISPIN=1,NSPIN
	DO JSPIN=1,NSPIN
	SPMAP(ISPIN,JSPIN)=0.D0
	SVMAP(ISPIN,JSPIN)=0.D0
	ENDDO
	SPMAP(ISPIN,ISPIN)=1.D0
	SVMAP(ISPIN,ISPIN)=1.D0
	ENDDO
	ELSEIF(NSPIN.EQ.2.AND.NSPIN0.EQ.1)THEN
	SPMAP(1,1)=0.5D0
	SPMAP(NSPIN,1)=0.5D0
	SVMAP(1,1)=1.D0 
	SVMAP(NSPIN,1)=1.D0 
	ELSEIF(NSPIN.EQ.1.AND.NSPIN0.EQ.2)THEN
	SPMAP(1,1)=1.0D0
	SPMAP(1,NSPIN0)=1.0D0
	SVMAP(1,1)=0.5D0
	SVMAP(1,NSPIN0)=0.5D0
	ENDIF
C *********************************************************
C TRY TO DEFINE WHETHER THE LATTICE IS THE SAME
	SAME=.TRUE.
	IF(NATOM.NE.NATOM0)SAME=.FALSE.
	IF(NSORT.NE.NSORT0)SAME=.FALSE.
	IF(SAME)THEN
	DO IATOM=1,NATOM
	IF(IS(IATOM).NE.IS0(IATOM))SAME=.FALSE.
	ENDDO
	DO ISORT=1,NSORT
	IF(NINT(ZNUC(ISORT)).NE.NINT(ZNUC0(ISORT)))
     &CALL READSCF_MESG01
	ENDDO
	ENDIF
C If SAME still survives, define mapping arrays NOW
	IF(SAME)THEN
	DO IATOM=1,NATOM
	DO JATOM=1,NATOM
	ATMAP(IATOM,JATOM)=0.D0
	ENDDO
	ATMAP(IATOM,IATOM)=1.D0
	ENDDO
	DO ISORT=1,NSORT
	DO JSORT=1,NSORT
	SOMAP(ISORT,JSORT)=0.D0
	ENDDO
	SOMAP(ISORT,ISORT)=1.D0
	ENDDO
C If not same structure, try to define mapping
	ELSEIF(.NOT.SAME.AND.NATOM.GT.NATOM0)THEN
C **********************************************************
C MAKING SUPERCELL STRUCTURE
	IF(ABS(DBLE(NATOM)/DBLE(NATOM0)-DBLE(NATOM/NATOM0)).
     &GT.1.D-10)CALL READSCF_MESG02
	NSUP=NATOM/NATOM0
C Try to find correspondence between new and old atomic positions
	DO IATOM=1,NATOM
	ISORT=IS(IATOM)
C Reduce new position to the old cell
	CALL SITEGEN_REDC(RBAS0E(1,1),RBAS0E(1,2),RBAS0E(1,3),
     &TAO1E(1,IATOM),TR1,1)
C Try to find it in the old basis
	DO IATOM0=1,NATOM0
	ISORT0=IS0(IATOM0)
	ATMAP(IATOM,IATOM0)=0.D0
	CALL SITEGEN_REDC(RBAS0(1,1),RBAS0(1,2),RBAS0(1,3),
     &TAO0E(1,IATOM0),TR0,1)
      F1=ABS(TR1(1)-TR0(1))
      F2=ABS(F1-1.D0)
	IF(F1.LE.1.D-05.OR.F2.LE.1.D-05)THEN
      F1=ABS(TR1(2)-TR0(2))
      F2=ABS(F1-1.D0)
	IF(F1.LE.1.D-05.OR.F2.LE.1.D-05)THEN
      F1=ABS(TR1(3)-TR0(3))
      F2=ABS(F1-1.D0)
	IF(F1.LE.1.D-05.OR.F2.LE.1.D-05)THEN
C Reduction exists, check charges:
	IF(NINT(ZNUC(ISORT)).EQ.NINT(ZNUC0(ISORT0)))THEN
	ATMAP(IATOM,IATOM0)=1.D0
	ENDIF
	ENDIF
	ENDIF
	ENDIF
	ENDDO
	ENDDO
C Check for errors:
C Every new atom can only come from one old atom!
	DO IATOM=1,NATOM
	CHK=0.D0
	DO IATOM0=1,NATOM0
	CHK=CHK+ATMAP(IATOM,IATOM0)
	ENDDO
	IF(ABS(CHK-1.D0).GT.1.D-10)
     &CALL READSCF_MESG02
	ENDDO
C Every old atom produces NSUP new atoms
	DO IATOM0=1,NATOM0
	CHK=0.D0
	DO IATOM=1,NATOM
	CHK=CHK+ATMAP(IATOM,IATOM0)
	ENDDO
	IF(ABS(CHK-DBLE(NSUP)).GT.1.D-10)
     &CALL READSCF_MESG02
	ENDDO
C Set correspondence between sorts
	DO ISORT=1,NSORT
	DO ISORT0=1,NSORT0
	SOMAP(ISORT,ISORT0)=0.D0
	ENDDO
	ENDDO
	DO IATOM=1,NATOM
	ISORT=IS(IATOM)
	DO IATOM0=1,NATOM0
	ISORT0=IS0(IATOM0)
	SOMAP(ISORT,ISORT0)=ATMAP(IATOM,IATOM0)
	ENDDO
	ENDDO
C **********************************************************
C SHRINKING THE CELL CASE
	ELSEIF(.NOT.SAME.AND.NATOM.LT.NATOM0)THEN
	IF(ABS(DBLE(NATOM0)/DBLE(NATOM)-DBLE(NATOM0/NATOM)).
     &GT.1.D-10)CALL READSCF_MESG02
	NSHR=NATOM0/NATOM
C Try to find correspondence between new and old atomic positions
      DO IATOM0=1,NATOM0
      ISORT0=IS0(IATOM0)
C Reduce old position to the new cell
      CALL SITEGEN_REDC(RBAS1E(1,1),RBAS1E(1,2),RBAS1E(1,3),
     &TAO0E(1,IATOM0),TR0,1)
C Try to find it in the new basis
      DO IATOM=1,NATOM
      ISORT=IS(IATOM)
      ATMAP(IATOM,IATOM0)=0.D0
      CALL SITEGEN_REDC(RBAS1E(1,1),RBAS1E(1,2),RBAS1E(1,3),
     &TAO1E(1,IATOM),TR1,1)
      F1=ABS(TR1(1)-TR0(1))
      F2=ABS(F1-1.D0)
      IF(F1.LE.1.D-05.OR.F2.LE.1.D-05)THEN
      F1=ABS(TR1(2)-TR0(2))
      F2=ABS(F1-1.D0)
      IF(F1.LE.1.D-05.OR.F2.LE.1.D-05)THEN
      F1=ABS(TR1(3)-TR0(3))
      F2=ABS(F1-1.D0)
      IF(F1.LE.1.D-05.OR.F2.LE.1.D-05)THEN
C Reduction exists, check charges:
      IF(NINT(ZNUC(ISORT)).EQ.NINT(ZNUC0(ISORT0)))THEN
      ATMAP(IATOM,IATOM0)=1.D0/DBLE(NSHR)
      ENDIF
      ENDIF
      ENDIF
      ENDIF
	ENDDO
	ENDDO
C Check for errors:
C Every new atom should appear NSHR times
      DO IATOM=1,NATOM
      CHK=0.D0
      DO IATOM0=1,NATOM0
      CHK=CHK+ATMAP(IATOM,IATOM0)
      ENDDO
      IF(ABS(CHK-1.D0).GT.1.D-10)
     &CALL READSCF_MESG02
      ENDDO
C Every old atom corresponds to only one new atom
      DO IATOM0=1,NATOM0
      CHK=0.D0
      DO IATOM=1,NATOM
      CHK=CHK+ATMAP(IATOM,IATOM0)
      ENDDO
      IF(ABS(CHK-1.D0/DBLE(NSHR)).GT.1.D-10)
     &CALL READSCF_MESG02
      ENDDO
C Set correspondence between sorts
      DO IATOM=1,NATOM
      ISORT=IS(IATOM)
      DO IATOM0=1,NATOM0
      ISORT0=IS0(IATOM0)
      SOMAP(ISORT,ISORT0)=ATMAP(IATOM,IATOM0)
      ENDDO
      ENDDO
      DO ISORT=1,NSORT
	SN=0.D0
      DO ISORT0=1,NSORT0
      SN=SN+SOMAP(ISORT,ISORT0)
      ENDDO
	IF(ABS(SN).LE.1.D-10)CALL READSCF_MESG02
      DO ISORT0=1,NSORT0
      SOMAP(ISORT,ISORT0)=SOMAP(ISORT,ISORT0)/SN
      ENDDO
      ENDDO
C **********************************************************
C NUMBER OF ATOMS IS THE SAME, SORTS ARE DIFFERENT
	ELSEIF(.NOT.SAME.AND.NATOM.EQ.NATOM0.AND.NSORT.NE.NSORT0)THEN
      DO IATOM=1,NATOM
      DO JATOM=1,NATOM
      ATMAP(IATOM,JATOM)=0.D0
      ENDDO
      ATMAP(IATOM,IATOM)=1.D0
      ENDDO
	IF(NSORT.GT.NSORT0)THEN
C Set correspondence between sorts
      DO ISORT=1,NSORT
      DO ISORT0=1,NSORT0
      SOMAP(ISORT,ISORT0)=0.D0
      ENDDO
      ENDDO
      DO IATOM=1,NATOM
      ISORT=IS(IATOM)
      DO IATOM0=1,NATOM0
      ISORT0=IS0(IATOM0)
      SOMAP(ISORT,ISORT0)=ATMAP(IATOM,IATOM0)
      ENDDO
      ENDDO
	ELSEIF(NSORT.LT.NSORT0)THEN
C Set correspondence between sorts
      DO IATOM=1,NATOM
      ISORT=IS(IATOM)
      DO IATOM0=1,NATOM0
      ISORT0=IS0(IATOM0)
      SOMAP(ISORT,ISORT0)=ATMAP(IATOM,IATOM0)
      ENDDO
      ENDDO
      DO ISORT=1,NSORT
      SN=0.D0
      DO ISORT0=1,NSORT0
      SN=SN+SOMAP(ISORT,ISORT0)
      ENDDO
      IF(ABS(SN).LE.1.D-10)CALL READSCF_MESG02
      DO ISORT0=1,NSORT0
      SOMAP(ISORT,ISORT0)=SOMAP(ISORT,ISORT0)/SN
      ENDDO
      ENDDO
	ENDIF
C **********************************************************
C NUMBER OF ATOMS IS THE SAME, SORTS ARE DIFFERENT
      ELSEIF(.NOT.SAME.AND.NATOM.EQ.NATOM0.AND.NSORT.EQ.NSORT0)THEN
      DO IATOM=1,NATOM
      DO JATOM=1,NATOM
      ATMAP(IATOM,JATOM)=0.D0
      ENDDO
      ATMAP(IATOM,IATOM)=1.D0
      ENDDO
C Set correspondence between sorts
	DO ISORT=1,NSORT
	DO ISORT0=1,NSORT0
	SOMAP(ISORT,ISORT0)=0.D0
	ENDDO
	ENDDO
      DO IATOM=1,NATOM
      ISORT=IS(IATOM)
      DO IATOM0=1,NATOM0
      ISORT0=IS0(IATOM0)
      SOMAP(ISORT,ISORT0)=SOMAP(ISORT,ISORT0)+ATMAP(IATOM,IATOM0)
      ENDDO
      ENDDO
      DO ISORT=1,NSORT
      SN=0.D0
      DO ISORT0=1,NSORT0
      SN=SN+SOMAP(ISORT,ISORT0)
      ENDDO
      IF(ABS(SN).LE.1.D-10)CALL READSCF_MESG02
      DO ISORT0=1,NSORT0
      SOMAP(ISORT,ISORT0)=SOMAP(ISORT,ISORT0)/SN
      ENDDO
      ENDDO
C *************************************************************
	ELSE
	CALL READSCF_MESG02
	ENDIF
C *************************************************************
C SET MAGNETIZATION MAPPING
C Transformation matrix my->xyz
	TD(-1,1)=(+1.D0,+0.D0)/SQRT(2.D0)
	TD(-1,2)=(+0.D0,+1.D0)/SQRT(2.D0)
	TD(-1,3)=(+0.D0,+0.D0)
	TD( 0,1)=(+0.D0,+0.D0)
	TD( 0,2)=(+0.D0,+0.D0)
	TD( 0,3)=(+1.D0,+0.D0)
	TD(+1,1)=(-1.D0,+0.D0)/SQRT(2.D0)
	TD(+1,2)=(+0.D0,+1.D0)/SQRT(2.D0)
	TD(+1,3)=(+0.D0,+0.D0)
C Reciprocal transformation matrix my->xyz
	TR(1,-1)=(+1.D0,+0.D0)/SQRT(2.D0)
	TR(1, 0)=(+0.D0,+0.D0)
	TR(1,+1)=(-1.D0,+0.D0)/SQRT(2.D0)
	TR(2,-1)=(+0.D0,-1.D0)/SQRT(2.D0)
	TR(2, 0)=(+0.D0,+0.D0)
	TR(2,+1)=(+0.D0,-1.D0)/SQRT(2.D0)
	TR(3,-1)=(+0.D0,+0.D0)
	TR(3, 0)=(+1.D0,+0.D0)
	TR(3,+1)=(+0.D0,+0.D0)
C UpDn to M transformation
	GD(1,-1)=(+0.0D0,0.D0)
	GD(1, 0)=(+0.5D0,0.D0)
	GD(1,+1)=(+0.0D0,0.D0)
	GD(1, 2)=(+0.5D0,0.D0)
	GD(2,-1)=(+0.0D0,0.D0)
	GD(2, 0)=(-0.5D0,0.D0)
	GD(2,+1)=(+0.0D0,0.D0)
	GD(2, 2)=(+0.5D0,0.D0)
	GD(3,-1)=(+1.0D0,0.D0)
	GD(3, 0)=(+0.0D0,0.D0)
	GD(3,+1)=(+0.0D0,0.D0)
	GD(3, 2)=(+0.0D0,0.D0)
	GD(4,-1)=(+0.0D0,0.D0)
	GD(4, 0)=(+0.0D0,0.D0)
	GD(4,+1)=(+1.0D0,0.D0)
	GD(4, 2)=(+0.0D0,0.D0)
C UpDn to M reciprocal transformation
	GR(-1,1)=(+0.0D0,0.D0)
	GR(-1,2)=(+0.0D0,0.D0)
	GR(-1,3)=(+1.0D0,0.D0)
	GR(-1,4)=(+0.0D0,0.D0)
	GR( 0,1)=(+1.0D0,0.D0)
	GR( 0,2)=(-1.0D0,0.D0)
	GR( 0,3)=(+0.0D0,0.D0)
	GR( 0,4)=(+0.0D0,0.D0)
	GR(+1,1)=(+0.0D0,0.D0)
	GR(+1,2)=(+0.0D0,0.D0)
	GR(+1,3)=(+0.0D0,0.D0)
	GR(+1,4)=(+1.0D0,0.D0)
	GR( 2,1)=(+1.0D0,0.D0)
	GR( 2,2)=(+1.0D0,0.D0)
	GR( 2,3)=(+0.0D0,0.D0)
	GR( 2,4)=(+0.0D0,0.D0)
	DO ISORT=1,NSORT                        !!! over sorts
	CALL ROTAXES(AXMAG(1,ISORT),TETA,PFI)
C New system transformation: M{xyz}=TRS*M{teta,fi,rho}
	TRN(1,1)=+DCMPLX(COS(PFI)*COS(TETA),0.D0)
	TRN(1,2)=-DCMPLX(SIN(PFI),0.D0)
	TRN(1,3)=+DCMPLX(SIN(TETA)*COS(PFI),0.D0)
	TRN(2,1)=+DCMPLX(SIN(PFI)*COS(TETA),0.D0)
	TRN(2,2)=+DCMPLX(COS(PFI),0.D0)
	TRN(2,3)=+DCMPLX(SIN(TETA)*SIN(PFI),0.D0)
	TRN(3,1)=-DCMPLX(SIN(TETA),0.D0)
	TRN(3,2)=+DCMPLX(0.D0,0.D0)
	TRN(3,3)=+DCMPLX(COS(TETA),0.D0)
	DO ISORT0=1,NSORT0
	IF(SOMAP(ISORT,ISORT0).GT.1.D-10)EXIT
	ENDDO
	IF(ISORT0.EQ.NSORT0+1)
     &CALL FULLSTOP('<ReadSCF>: Wrong sort pointer')
      CALL ROTAXES(AXMAG0(1,ISORT0),TETA,PFI)
C Old system transformation: M{xyz}=TRS*M{teta,fi,rho}
	TRO(1,1)=+DCMPLX(COS(PFI)*COS(TETA),0.D0)
	TRO(1,2)=-DCMPLX(SIN(PFI),0.D0)
	TRO(1,3)=+DCMPLX(SIN(TETA)*COS(PFI),0.D0)
	TRO(2,1)=+DCMPLX(SIN(PFI)*COS(TETA),0.D0)
	TRO(2,2)=+DCMPLX(COS(PFI),0.D0)
	TRO(2,3)=+DCMPLX(SIN(TETA)*SIN(PFI),0.D0)
	TRO(3,1)=-DCMPLX(SIN(TETA),0.D0)
	TRO(3,2)=+DCMPLX(0.D0,0.D0)
	TRO(3,3)=+DCMPLX(COS(TETA),0.D0)
C Transformation matrix is Tnew*Told^{+}
	DO I=1,3
	DO J=1,3
	TRC(I,J)=(0.D0,0.D0)
	DO K=1,3
	TRC(I,J)=TRC(I,J)+TRN(I,K)*DCONJG(TRO(J,K))
	ENDDO
	ENDDO
	ENDDO
C Transform to my-representation
	DO MY=-1,1
	DO NY=-1,1
	TRM(MY,NY)=(0.D0,0.D0)
	DO I=1,3
	DO J=1,3
	TRM(MY,NY)=TRM(MY,NY)+TD(MY,I)*TRC(I,J)*TR(J,NY)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
C Add density element
	DO MY=-1,1
	TRM(MY,2)=(0.D0,0.D0)
	TRM(2,MY)=(0.D0,0.D0)
	ENDDO
	TRM(2,2)=(+1.D0,0.D0)
C Transformation between UpDn vectors
	DO I=1,4
	DO J=1,4
	TUD(I,J)=(0.D0,0.D0)
	DO MY=-1,2
	DO NY=-1,2
	TUD(I,J)=TUD(I,J)+GD(I,MY)*TRM(MY,NY)*GR(NY,J)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
C TUD is what we really need:
	DO IORBS0=1,NORBS0
	DO IUPDN0=1,NSPIN0
	I0=IUPDN0+NSPIN0*(IORBS0-1)
	DO IORBS =1,NORBS
	DO IUPDN =1,NSPIN
	I=IUPDN+NSPIN*(IORBS-1)
	SRMAP(IUPDN,IORBS,IUPDN0,IORBS0,ISORT)=TUD(I,I0)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
C Override it for special cases, the same for int.region below
	    IF(NSPIN0.EQ.1.AND.NSPIN.EQ.2)THEN
	DO IUPDN=1,NSPIN
	SRMAP(IUPDN,NORBS,1,1,ISORT)=(0.0D0,0.D0)
	SRMAP(IUPDN,    1,1,1,ISORT)=(0.5D0,0.D0)
	ENDDO
	ELSEIF(NSPIN0.EQ.2.AND.NSPIN.EQ.1)THEN
	DO IUPDN0=1,NSPIN0
	SRMAP(1,1,IUPDN0,NORBS0,ISORT)=(0.D0,0.D0)
	SRMAP(1,1,IUPDN0,     1,ISORT)=(1.D0,0.D0)
	ENDDO
	ELSEIF(NSPIN0.EQ.1.AND.NSPIN.EQ.1)THEN
	SRMAP(1,1,1,1,ISORT)=(1.D0,0.D0)
	ENDIF
C *******************************************
C To see how it works, see example below
C Old Magnetization vector (Mx,My,Mz,Rho) related to AxMag0:
	ACOLD(1)=(0.D0,0.D0)
	ACOLD(2)=(0.D0,0.D0)
	ACOLD(3)=(0.D0,0.D0)
	ACOLD(4)=(5.D0,0.D0)
C Old Magnetization vector (M-,M0,M+,Rho)
	DO MY=-1,+1
	AMOLD(MY)=(0.D0,0.D0)
	DO I=1,3
	AMOLD(MY)=AMOLD(MY)+TD(MY,I)*ACOLD(I)
	ENDDO
	ENDDO
	AMOLD(2)=ACOLD(4)
C Old Magnetization vector (up,dn,+,-)
	DO IORBS0=1,NORBS0
	DO IUPDN0=1,NSPIN0
	ROOLD(IUPDN0,IORBS0)=(0.D0,0.D0)
	I0=IUPDN0+NSPIN0*(IORBS0-1)
	DO MY=-1,2
	ROOLD(IUPDN0,IORBS0)=ROOLD(IUPDN0,IORBS0)+GD(I0,MY)*AMOLD(MY)
	ENDDO
	ENDDO
	ENDDO
C Transform to new coordinates set by AXMAG:
	DO IORBS=1,NORBS
	DO IUPDN=1,NSPIN
	RONEW(IUPDN,IORBS)=(0.D0,0.D0)
	DO IORBS0=1,NORBS0
	DO IUPDN0=1,NSPIN0
	RONEW(IUPDN,IORBS)=RONEW(IUPDN,IORBS)+
     &SRMAP(IUPDN,IORBS,IUPDN0,IORBS0,ISORT)*ROOLD(IUPDN0,IORBS0)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
C New Magnetization vector (up,dn,+,-) transformed to (M-,M0,M+,rho)
	DO MY=-1,2
	AMNEW(MY)=(0.D0,0.D0)
	DO IORBS=1,NORBS
	DO IUPDN=1,NSPIN
	I=IUPDN+NSPIN*(IORBS-1)
	AMNEW(MY)=AMNEW(MY)+GR(MY,I)*RONEW(IUPDN,IORBS)
	ENDDO
	ENDDO
	ENDDO
C New Magnetization vector (M-,M0,M+,rho) transformed to (Mx,My,Mz,Rho)
	DO I=1,3
	ACNEW(I)=(0.D0,0.D0)
	DO MY=-1,+1
	ACNEW(I)=ACNEW(I)+TR(I,MY)*AMNEW(MY)
	ENDDO
	ENDDO
	ACNEW(4)=AMNEW(2)
	ENDDO                             !!! over sorts
C MAGNETIZATION MAPPING FOR THE INTERSTITIAL REGION
	DO IORBS=1,NORBS
	DO IUPDN=1,NSPIN
	DO IORBS0=1,NORBS0
	DO IUPDN0=1,NSPIN0
	SRMAP(IUPDN,IORBS,IUPDN0,IORBS0,0)=(0.D0,0.D0)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
C Check for same quantization axis for all atoms in
C old and new structures 
	ANX=AXMAG(1,1)
	ANY=AXMAG(2,1)
	ANZ=AXMAG(3,1)
	LA=1
	DO ISORT=1,NSORT
	IF(ABS(ANX-AXMAG(1,ISORT)).GE.1.D-07.OR.
     &   ABS(ANY-AXMAG(2,ISORT)).GE.1.D-07.OR.
     &   ABS(ANZ-AXMAG(3,ISORT)).GE.1.D-07)LA=0
	ENDDO
	ANX0=AXMAG0(1,1)
	ANY0=AXMAG0(2,1)
	ANZ0=AXMAG0(3,1)
	DO ISORT0=1,NSORT0
	IF(ABS(ANX0-AXMAG0(1,ISORT0)).GE.1.D-07.OR.
     &   ABS(ANY0-AXMAG0(2,ISORT0)).GE.1.D-07.OR.
     &   ABS(ANZ0-AXMAG0(3,ISORT0)).GE.1.D-07)LA=0
	ENDDO
C Check for same orientations
	LS=1
	DO ISORT=1,NSORT
	DO ISORT0=1,NSORT0
	IF(SOMAP(ISORT,ISORT0).GT.1.D-10)EXIT
	ENDDO
	IF(ABS(AXMAG(1,ISORT)-AXMAG0(1,ISORT0)).GE.1.D-07.OR.
     &   ABS(AXMAG(2,ISORT)-AXMAG0(2,ISORT0)).GE.1.D-07.OR.
     &   ABS(AXMAG(3,ISORT)-AXMAG0(3,ISORT0)).GE.1.D-07)LS=0
	ENDDO
C Unique axis for all atoms case
	IF(LA.EQ.1)THEN
	DO IORBS=1,NORBS
	DO IUPDN=1,NSPIN
	DO IORBS0=1,NORBS0
	DO IUPDN0=1,NSPIN0
	SRMAP(IUPDN,IORBS,IUPDN0,IORBS0,0)=
     &SRMAP(IUPDN,IORBS,IUPDN0,IORBS0,1)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ENDIF
C Orientations did not change case
	IF(LS.EQ.1)THEN
	DO IORBS=1,NORBS
	DO IUPDN=1,NSPIN
	DO IORBS0=1,NORBS0
	DO IUPDN0=1,NSPIN0
	IF(IORBS.EQ.IORBS0.AND.IUPDN.EQ.IUPDN0)
     &SRMAP(IUPDN,IORBS,IUPDN0,IORBS0,0)=(1.D0,0.D0)
	IF(IORBS.NE.IORBS0.OR.IUPDN.NE.IUPDN0)
     &SRMAP(IUPDN,IORBS,IUPDN0,IORBS0,0)=(0.D0,0.D0)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ENDIF
C If not two previous cases, leave only density
	IF(LA.EQ.0.AND.LS.EQ.0)THEN
	DO IUPDN=1,NSPIN
      SRMAP(IUPDN,    1,    1,NORBS0,0)=(0.D0,0.D0)
      SRMAP(IUPDN,    1,NSPIN,NORBS0,0)=(0.D0,0.D0)
      SRMAP(IUPDN,NORBS,    1,NORBS0,0)=(0.D0,0.D0)
      SRMAP(IUPDN,NORBS,NSPIN,NORBS0,0)=(0.D0,0.D0)
      SRMAP(IUPDN,NORBS,    1,     1,0)=(0.D0,0.D0)
      SRMAP(IUPDN,NORBS,NSPIN,     1,0)=(0.D0,0.D0)
      SRMAP(IUPDN,    1,    1,     1,0)=(0.5D0,0.D0)
      SRMAP(IUPDN,    1,NSPIN,     1,0)=(0.5D0,0.D0)
	ENDDO
	ENDIF
C Override above settings for special cases
	    IF(NSPIN0.EQ.1.AND.NSPIN.EQ.2)THEN
	DO IUPDN=1,NSPIN
	SRMAP(IUPDN,NORBS,1,1,0)=(0.0D0,0.D0)
	SRMAP(IUPDN,    1,1,1,0)=(0.5D0,0.D0)
	ENDDO
	ELSEIF(NSPIN0.EQ.2.AND.NSPIN.EQ.1)THEN
	DO IUPDN0=1,NSPIN0
	SRMAP(1,1,IUPDN0,NORBS0,0)=(0.D0,0.D0)
	SRMAP(1,1,IUPDN0,     1,0)=(1.D0,0.D0)
	ENDDO
	ELSEIF(NSPIN0.EQ.1.AND.NSPIN.EQ.1)THEN
	SRMAP(1,1,1,1,0)=(1.D0,0.D0)
	ENDIF
C *************************************************************
C CHECK MT-SPHERES IF THEY HAVE BEEN MADE
	IF(SMTMADE)THEN
	DO ISORT=1,NSORT
	SMT1(ISORT)=0.D0
	ENDDO
	DO ISORT0=1,NSORT0
	SMT0(ISORT0)=SMT0(ISORT0)*PAR/PARR0
	DO ISORT=1,NSORT
	SMT1(ISORT)=SMT1(ISORT)+SMT0(ISORT0)*SOMAP(ISORT,ISORT0)
	ENDDO
	ENDDO
	DO ISORT=1,NSORT
	IF(ABS(SMT(ISORT)-SMT1(ISORT)).GT.1.D-08)THEN
	WRITE(IUN,1010)
	WRITE(IUN,1020)SMT1(ISORT),TXTEL(ISORT)
	WRITE(IUN,1030)SMT(ISORT)
	ENDIF
	ENDDO
	ENDIF
C MEMORY DEALLOCATIONS
	DEALLOCATE(TAU0,TAO0,ZNUC0,SMT0,SMT1,AXMAG0,TAO1E,TAO0E)
1010	FORMAT(1X,'Warning from <READSCF>: MT-spheres mismatch')
1020	FORMAT(1X,'MT-sphere read   from SCFFILE =',G14.7,1X,A10)
1030	FORMAT(1X,'MT-sphere computed by MAKESMT =',G14.7)
	CONTAINS
C ERROR HANDLINGS
	SUBROUTINE READSCF_MESG01
	WRITE(IUT,*)'Warning from <READSCF>:'
	WRITE(IUT,*)'Nuclear charges in old and structures do not'
	WRITE(IUT,*)'coincide:',ZNUC(ISORT),ZNUC0(ISORT),TXTEL(ISORT)
	END SUBROUTINE READSCF_MESG01
	SUBROUTINE READSCF_MESG02
	WRITE(IUT,*)'Terminated in <READSCF>:'
	WRITE(IUT,*)'Cannot find an association between old and new'
	WRITE(IUT,*)'structures to built initial charge density'
	WRITE(IUT,*)'SCFFILE. Delete SCFFILE and restart the program.'
	WRITE(IUT,*)'An initial guess will be built automatically'
	WRITE(IUT,*)'as superposition of atomic densities.'
	CALL FULLSTOP(' ')
	END SUBROUTINE READSCF_MESG02
	END SUBROUTINE READSCF_MAPS



      SUBROUTINE READSCF_RHOS(NATOM0,NSORT0,NSPIN0,NORBS0,IS0,
     &           ATMAP,SOMAP,SPMAP,SVMAP,SRMAP,SAME)
************************************************************
*  Reads charge density destribution in cell               *
************************************************************
C SET ARRAY SIZE DATA
	USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_PARAM,   ONLY : NATOM,NSORT,NSPIN,NORBS
     &                       ,IS,ILST,ILSB,ILSV                   !!! from readini
      USE COM_DENDAT,  ONLY : RHO,RHOCOR,POTFRZ                   !!! from readscf
	USE COM_RADMESH, ONLY : NRAD,R                              !!! from makerad
	USE COM_SYMMLM,  ONLY : NSYM,LMSYM,SYM                      !!! from symmlm
      USE COM_LMTINP,  ONLY : SCSFILE                             !!! from readini
      USE COM_UNIT,    ONLY : IUN,IUR,IUT                         !!! from setup
C SET LOCAL VARIABLES
      IMPLICIT REAL*8 (A-H,O-Z)
	INTEGER, ALLOCATABLE :: NRAD0(:)       ! old radial mesh
	INTEGER, ALLOCATABLE :: NSYM0(:)       ! old symmetry
	INTEGER, ALLOCATABLE :: ILSV0(:)       ! old symmetry
	REAL*8,  ALLOCATABLE :: R0(:,:)        ! old radial grid
	REAL*8,  ALLOCATABLE :: RHOTR0(:)      ! temporary
	REAL*8,  ALLOCATABLE :: RHOTI0(:)      ! temporary
	REAL*8,  ALLOCATABLE :: RHOTR(:)       ! temporary
	REAL*8,  ALLOCATABLE :: RHOTI(:)       ! temporary
	REAL*8,  ALLOCATABLE :: RHOCOR0(:)     ! old core density
	REAL*8,  ALLOCATABLE :: POTFRZ0(:)     ! old spherical potential
	REAL*8,  ALLOCATABLE :: RHOT(:)        ! temporary
	REAL*8,  ALLOCATABLE :: POTT(:)        ! temporary
	REAL*8,  ALLOCATABLE :: DERINP(:,:)    ! input spline derivatives
	REAL*8,  ALLOCATABLE :: DEROUT(:,:)    ! output spline derivatives
	COMPLEX*16, ALLOCATABLE :: RHO0(:)     ! old charge density
	INTEGER IS0(NATOM0)
	REAL*8 ATMAP(NATOM,NATOM0),SOMAP(NSORT,NSORT0)
     &      ,SPMAP(NSPIN,NSPIN0),SVMAP(NSPIN,NSPIN0)
      COMPLEX*16 SRMAP(NSPIN,NORBS,NSPIN0,NORBS0,0:NSORT)
	LOGICAL SAME
      DATA PI/3.141592653589793D0/
C READ CONTROL DATA
	ALLOCATE(NRAD0(NSORT0))
	ALLOCATE(NSYM0(NATOM0))
	ALLOCATE(ILSV0(NSORT0))
	READ(1,*)
     &(NRAD0(ISORT0),ISORT0=1,NSORT0),
     &(NSYM0(IATOM0),IATOM0=1,NATOM0),
     &(ILSV0(ISORT0),ISORT0=1,NSORT0)
	NRADMEM0=0
	DO ISORT0=1,NSORT0
	NRADMEM0=MAX(NRADMEM0,NRAD0(ISORT0))
	ENDDO
C READ OLD RADIAL GRID
	ALLOCATE(R0(0:NRADMEM0,NSORT0))
	READ(1,*)((R0(IRAD0,ISORT0),IRAD0=0,NRAD0(ISORT0)),
     &ISORT0=1,NSORT0)
C READ CHARGE DENSITY
	ALLOCATE(RHO0(0:NRADMEM0))
	ALLOCATE(RHOTR0(0:NRADMEM0))
	ALLOCATE(RHOTI0(0:NRADMEM0))
	ALLOCATE(DERINP(0:NRADMEM0,2))
	ALLOCATE(RHOTR(0:NRADMEM))
	ALLOCATE(RHOTI(0:NRADMEM))
	ALLOCATE(DEROUT(0:NRADMEM,2))
	DO IATOM0=1,NATOM0                    !!! over old atoms
	ISORT0=IS0(IATOM0)
	DO IORBS0=1,NORBS0                    !!! over old iorbs
	DO IUPDN0=1,NSPIN0                    !!! over old spins
	DO ISYM0=1,NSYM0(IATOM0)              !!! over old symmetry
	READ(1,*)LM0
	READ(1,*)(RHO0(IRAD0),IRAD0=0,NRAD0(ISORT0))
C Interpolate to the new atomic positions
	DO IATOM=1,NATOM                      !!! over new atoms
	ISORT=IS(IATOM)
	DO IORBS=1,NORBS                      !!! over new spins
	DO IUPDN=1,NSPIN                      !!! over new spins
	IF(ABS(ATMAP(IATOM,IATOM0)).LE.1.D-10)CYCLE
	IF(ABS(SRMAP(IUPDN,IORBS,IUPDN0,IORBS0,ISORT)).LE.1.D-10)CYCLE
	DO L=0,ILSV(ISORT)                    !!! over new symmetry
	DO M=-L,L
	LM=L*(L+1)+M+1
	IF(.NOT.SYM(LM,IATOM))CYCLE
	IF(LM.NE.LM0)CYCLE
	ISYM=LMSYM(LM,IATOM)
      DO IRAD=0,NRAD(ISORT)
      IF(R(IRAD,ISORT)-R0(NRAD0(ISORT0),ISORT0).GT.1.D-09)EXIT
	ENDDO
      NR=IRAD-1
      DO IRAD0=0,NRAD0(ISORT0)
      RHOTR0(IRAD0)=DREAL(RHO0(IRAD0))
      RHOTI0(IRAD0)=DIMAG(RHO0(IRAD0))
	ENDDO
      CALL SPLIN3(R0(0,ISORT0),RHOTR0,DERINP,NRAD0(ISORT0)+1,
     &NRADMEM0+1,R(0,ISORT),RHOTR,DEROUT,NR+1,NRADMEM+1,-1)
      CALL SPLIN3(R0(0,ISORT0),RHOTI0,DERINP,NRAD0(ISORT0)+1,
     &NRADMEM0+1,R(0,ISORT),RHOTI,DEROUT,NR+1,NRADMEM+1,-1)
      DO IRAD=0,NR
      RHO(IRAD,ISYM,IATOM,IUPDN,IORBS)=
     &RHO(IRAD,ISYM,IATOM,IUPDN,IORBS)+
     &DCMPLX(RHOTR(IRAD),RHOTI(IRAD))*ATMAP(IATOM,IATOM0)*
     &SRMAP(IUPDN,IORBS,IUPDN0,IORBS0,ISORT)
	ENDDO
      DO IRAD=NR+1,NRAD(ISORT)
      RHO(IRAD,ISYM,IATOM,IUPDN,IORBS)=
     &RHO(NR  ,ISYM,IATOM,IUPDN,IORBS)
	ENDDO
      ENDDO                               !!! over m
      ENDDO                               !!! over l
      ENDDO                               !!! over new spins
      ENDDO                               !!! over new iorbs
      ENDDO                               !!! over new atoms
      ENDDO                               !!! over old symmetry
      ENDDO                               !!! over old spins
      ENDDO                               !!! over old iorbs
      ENDDO                               !!! over old atoms
C MEMORY DEALLOCATIONS
      DEALLOCATE(RHOTR0)
      DEALLOCATE(RHOTI0)
      DEALLOCATE(RHOTR)
      DEALLOCATE(RHOTI)
      DEALLOCATE(RHO0)
C READ CORE DENSITY
	ALLOCATE(RHOCOR0(0:NRADMEM0))
	ALLOCATE(POTFRZ0(0:NRADMEM0))
	ALLOCATE(RHOT(0:NRADMEM))
	ALLOCATE(POTT(0:NRADMEM))
	DO ISORT0=1,NSORT0              !!! over old sorts
	DO ISPIN0=1,NSPIN0              !!! over old spins
	READ(1,*)(RHOCOR0(IRAD0),IRAD0=0,NRAD0(ISORT0))
	READ(1,*)(POTFRZ0(IRAD0),IRAD0=0,NRAD0(ISORT0))
C Interpolate to the new atomic positions
	DO ISPIN=1,NSPIN                 !!! over spins
	DO ISORT=1,NSORT                 !!! over sorts
	IF(ABS(SOMAP(ISORT,ISORT0)).LE.1.D-10)CYCLE
	IF(ABS(SPMAP(ISPIN,ISPIN0)).LE.1.D-10)CYCLE
      DO IRAD=0,NRAD(ISORT)
      IF(R(IRAD,ISORT)-R0(NRAD0(ISORT0),ISORT0).GT.1.D-09)EXIT
      ENDDO
      NR=IRAD-1
      CALL SPLIN3(R0(0,ISORT0),RHOCOR0,DERINP,NRAD0(ISORT0)+1,
     &NRADMEM0+1,R(0,ISORT),RHOT,DEROUT,NR+1,NRADMEM+1,-1)
      CALL SPLIN3(R0(0,ISORT0),POTFRZ0,DERINP,NRAD0(ISORT0)+1,
     &NRADMEM0+1,R(0,ISORT),POTT,DEROUT,NR+1,NRADMEM+1,-1)
      DO IRAD=0,NR
      RHOCOR(IRAD,ISORT,ISPIN)=RHOCOR(IRAD,ISORT,ISPIN)+
     &RHOT(IRAD)*SOMAP(ISORT,ISORT0)*SPMAP(ISPIN,ISPIN0)
      POTFRZ(IRAD,ISORT,ISPIN)=POTFRZ(IRAD,ISORT,ISPIN)+
     &POTT(IRAD)*SOMAP(ISORT,ISORT0)*SVMAP(ISPIN,ISPIN0)
      ENDDO
      DO IRAD=NR+1,NRAD(ISORT)
      RHOCOR(IRAD,ISORT,ISPIN)=RHOCOR(NR,ISORT,ISPIN)
      POTFRZ(IRAD,ISORT,ISPIN)=POTFRZ(NR,ISORT,ISPIN)
      ENDDO
      ENDDO                                !!! over new sorts
      ENDDO                                !!! over new spins
      ENDDO                                !!! over old sorts
      ENDDO                                !!! over old spins
C MEMORY DEALLOCATIONS
      DEALLOCATE(R0,DERINP,DEROUT,RHOCOR0,POTFRZ0)
      DEALLOCATE(RHOT,POTT,NRAD0,NSYM0,ILSV0)
	END SUBROUTINE READSCF_RHOS





      SUBROUTINE READSCF_RHOI(NATOM0,NSORT0,NSPIN0,NORBS0,IS0,
     &           ATMAP,SOMAP,SPMAP,SVMAP,SRMAP,SAME)
************************************************************
*  Reads pseudodensity.                                    *
************************************************************
C SET ARRAY SIZE DATA
	USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_PARAM,   ONLY : NATOM,NSORT,NSPIN,NORBS,IS          !!! from readini
	USE COM_LATPAR,  ONLY : PAR,ORTH,OMEGAC,STRAIN              !!! from readini
	USE COM_LATTICE, ONLY : RBAS,GBAS,TAU,TAO                   !!! from readstr
	USE COM_FFTMESH, ONLY : NFFTG,AD,RD                         !!! from makefft
      USE COM_DENDAT,  ONLY : RHOPS                               !!! from readscf
	USE COM_PLWMESH, ONLY : NPLW,GPLW                           !!! from makeplw
      USE COM_UNIT,    ONLY : IUN,IUR,IUT                         !!! from setup
C SET LOCAL VARIABLES
      IMPLICIT REAL*8 (A-H,O-Z)
	REAL*8, ALLOCATABLE :: TAO0(:,:)       ! old basis
	REAL*8, ALLOCATABLE :: TAU0(:,:)       ! old basis
	REAL*8, ALLOCATABLE :: GPLW0(:,:)      ! old plane waves
	COMPLEX*16, ALLOCATABLE :: RHOPS0(:)   ! old pseudodensity
	COMPLEX*16, ALLOCATABLE :: RHOPS1(:)   ! old pseudodensity
	INTEGER IS0(NATOM0),NFFTG0(3)
	REAL*8 ATMAP(NATOM,NATOM0),SOMAP(NSORT,NSORT0)
     &      ,SPMAP(NSPIN,NSPIN0),SVMAP(NSPIN,NSPIN0)
      COMPLEX*16 SRMAP(NSPIN,NORBS,NSPIN0,NORBS0,0:NSORT)
	DIMENSION RBAS0(3,3),GBAS0(3,3),ORTH0(2:3),STRAIN0(3,3,2)
     &      ,G1(3),G0(3)
      LOGICAL SAME
      DATA PI/3.141592653589793D0/
C READ CONTROL DATA
	ALLOCATE(TAO0(3,NATOM0))
	ALLOCATE(TAU0(3,NATOM0))
	READ(1,*)RBAS0,GBAS0,ORTH0,PARR0,STRAIN0,NPLW0,NFFTG0,
     &((TAO0(I,IATOM0),I=1,3),IATOM0=1,NATOM0),
     &((TAU0(I,IATOM0),I=1,3),IATOM0=1,NATOM0)
	IF(NPLW0.NE.-1)THEN
C READ PLANE WAVES
	ALLOCATE(GPLW0(3,0:NPLW0))
	READ(1,*)((GPLW0(I,IPLW0),I=1,3),IPLW0=0,NPLW0)
C READ PSEUDODENSITY
	ALLOCATE(RHOPS0(0:NPLW0))
	ALLOCATE(RHOPS1(0:NPLW0))
	DO IORBS0=1,NORBS0                !!! over old iorbs
	DO IUPDN0=1,NSPIN0                !!! over old spins
	READ(1,*)(RHOPS0(IPLW0),IPLW0=0,NPLW0)
	READ(1,*)(RHOPS1(IPLW0),IPLW0=0,NPLW0)
	DO IORBS=1,NORBS                  !!! over new iorbs
	DO IUPDN=1,NSPIN                  !!! over new spins
	IF(ABS(SRMAP(IUPDN,IORBS,IUPDN0,IORBS0,0)).LE.1.D-10)CYCLE
      NPLWI=-1
      DO IPLW=0,NPLW
C SET GPLW BEFORE STRAIN
      DO I=1,3
      G1(I)=0.D0
      DO J=1,3
      G1(I)=G1(I)+GPLW(J,IPLW)*STRAIN(J,I,1)
      ENDDO
      ENDDO
      DO IPLW00=-1,NPLW0
	IF(IPLW00.EQ.-1)IPLW0=IPLW ! check this first
	IF(IPLW00.NE.-1)IPLW0=IPLW00
	IF(IPLW0.GT.NPLW0)CYCLE    ! do not check
	IF(IPLW00.EQ.IPLW)CYCLE    ! was checked
C SET GPLW0 BEFORE STRAIN
      DO I=1,3
      G0(I)=0.D0
      DO J=1,3
      G0(I)=G0(I)+GPLW0(J,IPLW0)*STRAIN0(J,I,1)
      ENDDO
      ENDDO
	ABSD=(G1(1)-G0(1))**2+(G1(2)-G0(2))**2+(G1(3)-G0(3))**2
      IF(ABSD.LE.1.D-05)THEN
      RHOPS(IPLW,IUPDN,IORBS)=RHOPS(IPLW,IUPDN,IORBS)+
     &RHOPS0(IPLW0)*SRMAP(IUPDN,IORBS,IUPDN0,IORBS0,0)
      NPLWI=NPLWI+1
	EXIT
      ENDIF
	ENDDO
      ENDDO
	ENDDO                                  !!! over new spins
      ENDDO                                  !!! over new iorbs
      ENDDO                                  !!! over old spins
      ENDDO                                  !!! over old iorbs
	DEALLOCATE(TAU0,GPLW0,RHOPS0,RHOPS1)
      WRITE(IUN,1010)NPLW0,NPLW,NPLWI
 1010 FORMAT(1X,'Plane waves: old=',I7,'; new=',I7,'; coinciding ',I7)
C No plane wave expansions, skip records
	ELSEIF(NPLW0.EQ.-1)THEN
	READ(1,*)
	DO IORBS0=1,NORBS0
	DO ISPIN0=1,NSPIN0
	READ(1,*)
	READ(1,*)
	ENDDO
	ENDDO
	ENDIF
	END SUBROUTINE READSCF_RHOI





      SUBROUTINE READSCF_REST(NATOM0,NSORT0,NSPIN0,NORBS0,IS0,
     &             ATMAP,SOMAP,SPMAP,SVMAP,SRMAP,SAME)
************************************************************
*  Reads restart information.                              *
************************************************************
C SET ARRAY SIZE DATA
	USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_CONTROL ,ONLY: LRWF                                !!! from readini
      USE COM_PARAM   ,ONLY: NATOM,NSORT,NSPIN,NORBS
     &                      ,IS,ILST,ILSB,ILSV                   !!! from readini
      USE COM_KAPPAR  ,ONLY: NPAN,NKAP,EKAP                      !!! from readini
      USE COM_INPPAR  ,ONLY: ENY,DNY,NODES,MNY                   !!! from readini
      USE COM_UNIT    ,ONLY: IUN,IUR,IUT                         !!! from setup
C SET LOCAL VARIABLES
      IMPLICIT REAL*8 (A-H,O-Z)
	INTEGER, ALLOCATABLE :: ILST0(:)        ! old LmaxT
	INTEGER, ALLOCATABLE :: ILSB0(:)        ! old LmaxT
	INTEGER, ALLOCATABLE :: NOD0(:,:,:)     ! old Nodes
	INTEGER, ALLOCATABLE :: MNY0(:,:,:)     ! old Mnus
	REAL*8,  ALLOCATABLE :: ENY0(:,:,:,:)   ! old Enus
	REAL*8,  ALLOCATABLE :: DNY0(:,:,:,:)   ! old Dnus
	REAL*8,  ALLOCATABLE :: ENY1(:,:,:,:)   ! tmp Enus
	REAL*8,  ALLOCATABLE :: DNY1(:,:,:,:)   ! tmp Dnus
	INTEGER IS0(NATOM0)
	REAL*8 ATMAP(NATOM,NATOM0),SOMAP(NSORT,NSORT0)
     &      ,SPMAP(NSPIN,NSPIN0),SVMAP(NSPIN,NSPIN0)
      COMPLEX*16 SRMAP(NSPIN,NORBS,NSPIN0,NORBS0,0:NSORT)
      LOGICAL SAME
      DATA PI/3.141592653589793D0/
C READ CONTROL DATA
	ALLOCATE(ILST0(NSORT0))
	ALLOCATE(ILSB0(NSORT0))
	READ(1,*)NKAP0,(ILST0(ISORT0),ISORT0=1,NSORT0),
     &  (ILSB0(ISORT0),ISORT0=1,NSORT0)
	LMAXT0=0
	LMAXB0=0
	DO ISORT0=1,NSORT0
	LMAXT0=MAX(ILST0(ISORT0),LMAXT0)
	LMAXB0=MAX(ILSB0(ISORT0),LMAXB0)
	ENDDO
C MEMORY ALLOCATIONS
	ALLOCATE(ENY0(0:LMAXT0,NSORT0,NKAP0,NSPIN0))
	ALLOCATE(DNY0(0:LMAXT0,NSORT0,NKAP0,NSPIN0))
	ALLOCATE(MNY0(0:LMAXT0,NSORT0,NKAP0))
	ALLOCATE(NOD0(0:LMAXT0,NSORT0,NKAP0))
	ALLOCATE(ENY1(0:LMAXTMEM,NSORTMEM,NKAPMEM,NSPINMEM))
	ALLOCATE(DNY1(0:LMAXTMEM,NSORTMEM,NKAPMEM,NSPINMEM))
	READ(1,*)
     &((((ENY0(L0,ISORT0,IKAP0,ISPIN0),L0=0,ILST0(ISORT0)),
     &ISORT0=1,NSORT0),IKAP0=1,NKAP0),ISPIN0=1,NSPIN0),
     &((((DNY0(L0,ISORT0,IKAP0,ISPIN0),L0=0,ILST0(ISORT0)),
     &ISORT0=1,NSORT0),IKAP0=1,NKAP0),ISPIN0=1,NSPIN0),
     &(((MNY0(L0,ISORT0,IKAP0),L0=0,ILST0(ISORT0)),
     &ISORT0=1,NSORT0),IKAP0=1,NKAP0),
     &(((NOD0(L0,ISORT0,IKAP0),L0=0,ILST0(ISORT0)),
     &ISORT0=1,NSORT0),IKAP0=1,NKAP0)
C TRY RESET ENYS
	DO ISPIN=1,NSPIN
	DO ISORT=1,NSORT
	DO IKAP=1,NKAP(1)
	DO L=0,ILST(ISORT)
	ENY1(L,ISORT,IKAP,ISPIN)=0.D0
	DNY1(L,ISORT,IKAP,ISPIN)=0.D0
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	DO 400 ISPIN0=1,NSPIN0
	DO 300 ISPIN=1,NSPIN
	IF(ABS(SPMAP(ISPIN,ISPIN0)).LE.1.D-10)GOTO300
	DO 200 ISORT0=1,NSORT0                    !!! over isort0
      DO 100 ISORT=1,NSORT
	IF(ABS(SOMAP(ISORT,ISORT0)).LE.1.D-10)GOTO100
      DO IKAP=1,NKAP(1)
      DO L=0,MIN(ILST(ISORT),ILST0(ISORT0))
	FKAP0=0.D0
	DO IKAP0=1,NKAP0                      !!! over ikap0
      IF((NODES(L,ISORT,IKAP,1).EQ.NOD0(L,ISORT0,IKAP0)).AND.
     &(MNY(L,ISORT,IKAP,1).EQ.3.OR.MNY(L,ISORT,IKAP,1).EQ.
     &MNY0(L,ISORT0,IKAP0)))FKAP0=FKAP0+1.D0
	ENDDO
      DO IKAP0=1,NKAP0
      IF((NODES(L,ISORT,IKAP,1).EQ.NOD0(L,ISORT0,IKAP0)).AND.
     &(MNY(L,ISORT,IKAP,1).EQ.3.OR.MNY(L,ISORT,IKAP,1).EQ.
     &MNY0(L,ISORT0,IKAP0)))THEN
      ENY1(L,ISORT,IKAP,ISPIN)=ENY1(L,ISORT,IKAP,ISPIN)+
     &ENY0(L,ISORT0,IKAP0,ISPIN0)*SOMAP(ISORT,ISORT0)*
     &SVMAP(ISPIN,ISPIN0)/FKAP0
      DNY1(L,ISORT,IKAP,ISPIN)=DNY1(L,ISORT,IKAP,ISPIN)+
     &DNY0(L,ISORT0,IKAP0,ISPIN0)*SOMAP(ISORT,ISORT0)*
     &SVMAP(ISPIN,ISPIN0)/FKAP0
	ENDIF
	ENDDO
	ENDDO
	ENDDO
 100	CONTINUE
 200	CONTINUE
 300	CONTINUE
 400	CONTINUE
	DO ISORT=1,NSORT
	DO IKAP=1,NKAP(1)
	DO L=0,ILST(ISORT)
	DO ISPIN=1,NSPIN
	IF(ABS(ENY1(L,ISORT,IKAP,ISPIN)).GE.1.D-10)
     &ENY(L,ISORT,IKAP,1,ISPIN)=ENY1(L,ISORT,IKAP,ISPIN)
	IF(ABS(DNY1(L,ISORT,IKAP,ISPIN)).GE.1.D-10)
     &DNY(L,ISORT,IKAP,1,ISPIN)=DNY1(L,ISORT,IKAP,ISPIN)
	ENDDO
C AVERAGE OVER SPINS IF REQUIRED
      IF(LRWF.EQ.1)THEN
      ENY(L,ISORT,IKAP,1,1)=(ENY(L,ISORT,IKAP,1,1)+
     &ENY(L,ISORT,IKAP,1,NSPIN))/2.D0
      DNY(L,ISORT,IKAP,1,1)=(DNY(L,ISORT,IKAP,1,1)+
     &DNY(L,ISORT,IKAP,1,NSPIN))/2.D0
      ENY(L,ISORT,IKAP,1,NSPIN)=ENY(L,ISORT,IKAP,1,1)
      DNY(L,ISORT,IKAP,1,NSPIN)=DNY(L,ISORT,IKAP,1,1)
      ENDIF
	ENDDO
      ENDDO
      ENDDO
C MEMORY DEALLOCATIONS
	DEALLOCATE(ENY0,DNY0,ENY1,DNY1,MNY0,NOD0,ILSB0,ILST0)
	END SUBROUTINE READSCF_REST
