	SUBROUTINE GETCOMB(COMB,NMAX)
*******************************************************
*  Evaluates combinatoric array C(i,n)=n!/i!/(n-i)!)  *
*******************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	DIMENSION COMB(0:NMAX,0:NMAX)
	REAL*8, ALLOCATABLE :: DF(:)
C MEMORY ALLOCATIONS
	ALLOCATE(DF(0:NMAX))
	DF(0)=1.D0
	DO I=1,NMAX
	DF(I)=DF(I-1)*DBLE(I)
	ENDDO
C  N>0
	DO N=0,NMAX
	DO I=0,N
	COMB(I,N)=DF(N)/DF(I)/DF(N-I)
	ENDDO
	DO I=N+1,NMAX
	COMB(I,N)=0.D0
	ENDDO
	ENDDO
C MEMORY DEALLOCATIONS
	DEALLOCATE(DF)
	END	

	SUBROUTINE GETCONF(NREP,LREP,MREP,ICNF,NCNF,LCNF)
***************************************************************
* Returns configuration LCNF{n1...ng} corresponding to Icnf   *
***************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
	INTEGER LREP(NREP),MREP(0:NREP),LCNF(NREP)
	ICUR=ICNF-1
	DO IREP=NREP,1,-1
	LCNF(IREP)=ICUR/MREP(IREP-1)
	ICUR=ICUR-LCNF(IREP)*MREP(IREP-1)
	ENDDO
	END

	SUBROUTINE SETCONF(NREP,LREP,MREP,ICNF,NCNF,LCNF)
***************************************************************
* Returns Icnf corresponding to configuration LCNF{n1...ng}   *
***************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
	INTEGER LREP(NREP),MREP(0:NREP),LCNF(NREP)
	ICNF=1
	DO IREP=1,NREP
	ICNF=ICNF+LCNF(IREP)*MREP(IREP-1)
	ENDDO
	END


	REAL*8 FUNCTION FERMIFUN(E,T)
*******************************************************
*  Evaluates Fermi function                           *
*******************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	IF(T.GT.0.D0)THEN
	FERMIFUN=1.D0/(SAFEEXP(E/T)+1.D0)
	ELSEIF(T.EQ.0.D0.AND.E.LT.0.D0)THEN
	FERMIFUN=1.D0
	ELSEIF(T.EQ.0.D0.AND.E.EQ.0.D0)THEN
	FERMIFUN=0.5D0
	ELSEIF(T.EQ.0.D0.AND.E.GT.0.D0)THEN
	FERMIFUN=0.D0
	ENDIF
	END




	REAL*8 FUNCTION SAFEEXP(POW)
*******************************************************
*  Safely evaluates expontent e(pow)                  *
*******************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	DATA POWMAX/60.D0/
	IF(POW.GT.POWMAX)THEN
	SAFEEXP=EXP(POWMAX)	
	ELSEIF(POW.LT.-POWMAX)THEN
	SAFEEXP=EXP(-POWMAX)
	ELSE
	SAFEEXP=EXP(POW)
	ENDIF
	END



	SUBROUTINE PARAROOT(F1,F2,F3,X1,X2,X3,RR1,RI1,RR2,RI2)
******************************************************* 
*   Returns zeroes of parabola                        *
*******************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	COMPLEX*16 R1,R2
C PARABOLIC INTERPOLATION FORMULA
C	F3FUN(X,F1,F2,F3,X1,X2,X3)=
C    &  (X-X2)*(X-X3)/(X1-X2)/(X1-X3)*F1+
C    &  (X-X1)*(X-X3)/(X2-X1)/(X2-X3)*F2+
C    &  (X-X1)*(X-X2)/(X3-X1)/(X3-X2)*F3
	IF(ABS(X1-X2).GT.1.D-10.AND.
     &   ABS(X1-X3).GT.1.D-10.AND.
     &   ABS(X2-X3).GT.1.D-10)THEN
	A=1.D0       /(X1-X2)/(X1-X3)*F1+
     &  1.D0       /(X2-X1)/(X2-X3)*F2+
     &  1.D0       /(X3-X1)/(X3-X2)*F3
	B=(-X2-X3)   /(X1-X2)/(X1-X3)*F1+
     &  (-X1-X3)   /(X2-X1)/(X2-X3)*F2+
     &  (-X1-X2)   /(X3-X1)/(X3-X2)*F3
	C=(-X2)*(-X3)/(X1-X2)/(X1-X3)*F1+
     &  (-X1)*(-X3)/(X2-X1)/(X2-X3)*F2+
     &  (-X1)*(-X2)/(X3-X1)/(X3-X2)*F3
	ELSEIF(ABS(X1-X2).LE.1.D-10)THEN
	A=0.D0
	B=(F1-F3) ! /(X1-X3)
	C=(F3*X1-F1*X3) ! /(X1-X3)
	ELSEIF(ABS(X1-X3).LE.1.D-10)THEN
	A=0.D0
	B=(F2-F3) ! /(X2-X3)
	C=(F3*X2-F2*X3) ! /(X2-X3)
	ELSEIF(ABS(X2-X3).LE.1.D-10)THEN
	A=0.D0
	B=(F1-F2) ! /(X1-X2)
	C=(F2*X1-F1*X2) ! /(X1-X2)
	ELSE
	CALL FULLSTOP('<PARAROOT>:: Degeneracy')
	ENDIF
C
	IF(ABS(A).GE.1.D-10)THEN
	R1=(-B+SQRT(DCMPLX(B**2-4.D0*A*C,0.D0)))/2.D0/A
	R2=(-B-SQRT(DCMPLX(B**2-4.D0*A*C,0.D0)))/2.D0/A
	ELSE
	IF(ABS(B).GE.1.D-10)
     &R1=DCMPLX(-C/B,0.D0)
	IF(ABS(B).LT.1.D-10)
     &R1=DCMPLX(1.D+10,0.D0)
	R2=DCMPLX(1.D+10,0.D0)
	ENDIF
	RR1=DREAL(R1)
	RR2=DREAL(R2)
	RI1=DIMAG(R1)
	RI2=DIMAG(R2)
	END


