	SUBROUTINE LMTSCF
************************************************************
*   The calculation of the energy band structure in terms  *
*   of LMTO method (O.K.Andersen,Phys.Rev.B12,3060(1975))  *
************************************************************
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                         !!! from setup
      USE COM_LMTINP,  ONLY : TRACINP                        !!! from readini
      USE COM_CONTROL, ONLY : LIFT,LMTO                      !!! from readini
      USE COM_SCFLOOP, ONLY : NITER1                         !!! from readini
      USE COM_CHANNELS,ONLY : IUSCF,IUPOT                    !!! from readini
	USE COM_LMTOUT,  ONLY : ICPOT                          !!! from readini
      USE COM_UNIT,    ONLY : IUN,IUR,IUT,IUI                !!! from setup
C SET LOCAL VARIABLES
	IMPLICIT REAL*8 (A-H,O-Z)
	CHARACTER ST*8,FN*8,PROGN*8
	LOGICAL EXIT,TRACER
	DATA ST/'started'/,FN/'finished'/
C CALCULATE STRUCTURE CONSTANTS
	progn='Strmsh  '
      if(tracer(progn,tracinp,iut,iur,iui))then
	call timel(times)
	write(iun,1110)progn,st,times,curmem,totmem
	CALL VECGEN         !!! generate structure vectors
	CALL STRSNG         !!! check for singularities
	CALL STRMSH         !!! calculate structure constants
	call timel(timef)
	write(iun,1120)progn,fn,timef,curmem,totmem
	endif
C CALCULATE SCREENED STRUCTURE CONSTANTS
	progn='Scrcon  '
      if(tracer(progn,tracinp,iut,iur,iui))then
      call timel(times)
      write(iun,1110)progn,st,times,curmem,totmem
      CALL ALPCON         !!! get screening parameters "alpha"
      CALL SCRCON         !!! screen bare structure constants
      call timel(timef)
      write(iun,1120)progn,fn,timef,curmem,totmem
	endif
	IF(LIFT.LE.1)CALL ARTOUT('No band structure calculation')
	CALL SPLACE(0)      !!! save Dny for admixing
C  *****  THE LOOP OF SELF-CONSISTENCY *****
	DO 100 ITER=1,NITER1
	write(iun,1100)iter
	CALL RHOFILES(IUSCF,'R','A')! read charge density
C CALCULATE MULTIPOLE FIELD
	CALL MULTRHO        !!! calculate moments of (pseudo)density
	CALL MULTFTR        !!! multipoles as lattice sums
	CALL MULTASA        !!! multipoles as lattice sums
	CALL PRINTC(1)      !!! save input charge for printing
      CALL MIXROB(ITER,1) !!! initialize Broyden mixing
      CALL MIXHUB(ITER,1) !!! initialize mixing for occupancies
C CALCULATE FULL POTENTIAL
	progn='FullPOT '
	CALL POTFILES(IUPOT,'!','A')! initialize full potential
      if(tracer(progn,tracinp,iut,iur,iui))then
	call timel(times)
	write(iun,1110)progn,st,times,curmem,totmem
	CALL VCOUL          !!! calculate Coulomb potential within MT-spheres
	CALL VEXCH(1)       !!! calculate ex.-cor.potential within MT-spheres
	CALL VCFTR          !!! calculate Coulomb potential in the int.region
	CALL VXFTR(1)       !!! calculate ex.-cor.potential in the int.region
	CALL ZEROV          !!! set up energy zero
	CALL VPINT          !!! calculate convolutions of pseudopotential
	CALL PRINT2         !!! print necessary information
      CALL HUBPOT         !!! adds Hubbard correction to the potential
      CALL PRINTH(IUN,3)  !!! print Constrained potential
      CALL PRINTH(IUN,4)  !!! print Hubbard correction
	call timel(timef)
	write(iun,1120)progn,fn,timef,curmem,totmem
	endif
	CALL RHOFILES(IUSCF,'!','D')! deallocate charge density
	IF(ICPOT.GT.0)
     &CALL POTFILES(IUPOT,'W','!')! store the potential if needed
C CALCULATE BAND STRUCTURE
	progn='Bands   '
      if(tracer(progn,tracinp,iut,iur,iui))then
	call timel(times)
	write(iun,1110)progn,st,times,curmem,totmem
	CALL BANDS          !!! find band energies and eigenvectors
	call timel(timef)
	write(iun,1120)progn,fn,timef,curmem,totmem
	endif
      IF(LIFT.LE.2)CALL ARTOUT('No SCF loop')
C CALCULATE FULL VALENCE PLUS SEMICORE DENSITY IN UNIT-CELL
	progn='FullRHO '
      if(tracer(progn,tracinp,iut,iur,iui))then
	call timel(times)
	write(iun,1110)progn,st,times,curmem,totmem
	CALL RHOFUL         !!! find new valence charge density within MTS
	CALL SRODOT         !!! find energy derivative for rho(r)
      CALL ORBMAG         !!! find orbital contribution to magnetization
	CALL EMADER         !!! find derivative for EMA
	CALL RENROV         !!! renormalize valence density
      CALL RHOHUB         !!! set occupation numbers
	call timel(timef)
	write(iun,1120)progn,fn,timef,curmem,totmem
	endif
C CALCULATE DEEP CORE
	progn='RenCOR  '
      if(tracer(progn,tracinp,iut,iur,iui))then
	call timel(times)
	write(iun,1110)progn,st,times,curmem,totmem
	CALL RENCOR(ITER)   !!! find core charge density destribution in cell
	call timel(timef)
	write(iun,1120)progn,fn,timef,curmem,totmem
	endif
	CALL POTFILES(IUPOT,'W','D')! store the potential
C CALCULATE TOTAL ENERGY AND FORCES
      CALL PRINTH(IUN,1)  !!! print occupation numbers
      CALL WRITEHUB(2)    !!! store occupation numbers
	progn='Energy  '
      if(tracer(progn,tracinp,iut,iur,iui))then
	call timel(times)
	write(iun,1110)progn,st,times,curmem,totmem
	CALL SENERGY(EXIT)  !!! calculate Etot in LDA or GGA91
      CALL SFORCES        !!! calculate HF+Pulay - forces 
	call timel(timef)
	write(iun,1120)progn,fn,timef,curmem,totmem
	endif
	CALL PRINTC(2)      !!! print input-output charge transfer
	IF(EXIT)CALL ARTOUT('Total energy convergence reached')
C ADMIX DENSITY FOR THE NEXT ITERATION
      CALL MIXROB(ITER,2) !!! mix densities according to Broyden
      CALL MIXRO1(ITER,EXIT)! mix densities with linear mixing
      CALL MIXHUB(ITER,2) !!! mix occupancies
	CALL SPLACE(ITER)   !!! place Eny at the centers of gravity
	CALL RHOFILES(IUSCF,'W','D')! store charge density at the iteration
	IF(EXIT)CALL ARTOUT('Charge density convergence reached')
	write(iun,1130)TOTMEM
	write(iun,1140)TXTMEM
 100  CONTINUE
	CALL ARTOUT('Number of iterations exceeded in the SCF1 loop')
1100	FORMAT(/'                ITERATION ',I3/)
1110  FORMAT('***** ',A8,1X,A8,'; CPU : ',G12.5,
     &       '; CUR/MAX mem.(Mb): ',G10.3,'/',G10.3)
1120  FORMAT('***** ',A8,1X,A8,'; CPU : ',G12.5,
     &       '; CUR/MAX mem.(Mb): ',G10.3,'/',G10.3)
1130  FORMAT(/' Maximum memory allocated (Mbyte) ==',G14.7)
1140  FORMAT( ' Peak memory reached in ',A)
	END
