      SUBROUTINE EXCVXC(ROUP,RODN,GRUP,GRDN,G2UP,G2DN,
     &ALUP,ALDN,IXC,IKEY,NSPIN,EXCUP,EXCDN,VXCUP,VXCDN)
*************************************************************
*  Calculates exchange-correlation energy and potential     *
*  after LSDA or GGA91/96 [see PRL 77, 3865 (1996)].        *
*    ROUP=Rho_up                                            *
*    RODN=Rho_dn                                            *
*    GRUP=Nabla_a(Rho_up)         (  a = x,y,z or -1,0,1)   *
*    GRDN=Nabla_a(Rho_dn)         (  a = x,y,z or -1,0,1)   *
*    G2UP=Nabla_a*Nabla_b(Rho_up) (a,b = x,y,z or -1,0,1)   *
*    G2DN=Nabla_a*Nabla_b(Rho_dn) (a,b = x,y,z or -1,0,1)   *
*    ALUP=Nabla^2(Rho_up) (Laplasian)                       *
*    ALDN=Nabla^2(Rho_dn) (Laplasian)                       *
*  The ILDA=IXC-MOD(IXC,10) is avaiable with the following  *
*  parametrizations:                                        *
*  ILDA=0 - no exchange-correlation                         *
*      =1 - U.von.Barth and L.Hedin,                        *
*           J.Phys.C5, 1629 (1972)                          *
*      =2 - O.E.Gunnarsson and S.Lundqvist                  *
*           Phys.Rev.B                                      *
*      =3 - V.L.Moruzzi, J.F.Janak, and A.R.Williams,       *
*           Calculated Electronic Properties of Metals      *
*           (New York, Pergamon Press, 1978)                *
*      =4 - S.H.Vosko, L.Wilk, and M.Nusair                 *
*           Can.J.Phys.58, 1200 (1980)                      *
*           parametrization of the Ceperly-Alder form:      *
*           D.M.Ceperly, B.J.Alder,                         *
*           Phys.Rev.Lett.45, 566 (1980)                    *
*      =5   J.Perdew, X.Wang (1991) programs of LSDA=4      *
*      =6 - Original Gaspar-Kohn-Sham (rho**1/3)            *
*  To get GGA91 with specific LSDA, set IXC=ILDA+10.        *
*  To get GGA96 with specific LSDA, set IXC=ILDA+20.        *
*************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	COMPLEX*16 
     &    GRUP(-1:1),GRUP2(-1:1),G2UP(-1:1,-1:1),G2UP2(-1:1,-1:1)
     &   ,GRDN(-1:1),GRDN2(-1:1),G2DN(-1:1,-1:1),G2DN2(-1:1,-1:1)
     &   ,DVX2UP(-1:1,-1:1),DVC2UP(-1:1,-1:1)
     &   ,DVX2DN(-1:1,-1:1),DVC2DN(-1:1,-1:1)
      DATA PI/3.141592653589793D0/
C  Set keys for LDA and GGA
	ILDA=MOD(IXC,10)     !!! LDA key
	IGGA=(IXC-ILDA)/10   !!! GGA key
	IF(ILDA.LT.0.OR.ILDA.GT.6)CALL FULLSTOP('<EXCVXC>:WRONG ILDA')
	IF(IGGA.LT.0.OR.IGGA.GT.2)CALL FULLSTOP('<EXCVXC>:WRONG IGGA')
C  Energies and potentials
	EXCUP=0.D0
	EXCDN=0.D0
	VXCUP=0.D0
	VXCDN=0.D0
	IF(ILDA.EQ.0)RETURN
C  ********************************
C  LOCAL SPIN DENSITY APPROXIMATION
C  ********************************
C  Local exchange
	EXLDAUP=0.D0
	VXLDAUP=0.D0
	EXLDADN=0.D0
	VXLDADN=0.D0
******
	CALL LSDEXCH(2.D0*ROUP,EXLDAUP,VXLDAUP)
	CALL LSDEXCH(2.D0*RODN,EXLDADN,VXLDADN)
******
	EXCUP=EXCUP+EXLDAUP
	EXCDN=EXCDN+EXLDADN
	VXCUP=VXCUP+VXLDAUP
	VXCDN=VXCDN+VXLDADN
C  Local correlation:
	ECLDA=0.D0
	VCLDAUP=0.D0
	VCLDADN=0.D0
******
      IF(ILDA.EQ.1.OR.ILDA.EQ.2.OR.ILDA.EQ.3)
     &CALL LSDCORB(ROUP,RODN,ECLDA,VCLDAUP,VCLDADN,ILDA)
      IF(ILDA.EQ.4)
     &CALL LSDCORV(ROUP,RODN,ECLDA,VCLDAUP,VCLDADN)
      IF(ILDA.EQ.5)
     &CALL LSDCORP(ROUP,RODN,ECLDA,VCLDAUP,VCLDADN)
******
	EXCUP=EXCUP+ECLDA
	EXCDN=EXCDN+ECLDA
	VXCUP=VXCUP+VCLDAUP
	VXCDN=VXCDN+VCLDADN
C  *****************************************************
C  GENERALIZED GRADIENT APPROXIMATION AFTER PERDEW ET.AL
C  *****************************************************
C  Non-local exchange:
	DO MY=-1,1
	GRUP2(MY)=2.D0*GRUP(MY)
	GRDN2(MY)=2.D0*GRDN(MY)
	DO MY1=-1,1
	G2UP2(MY1,MY)=2.D0*G2UP(MY1,MY)
	G2DN2(MY1,MY)=2.D0*G2DN(MY1,MY)
	ENDDO
	ENDDO
	EXGGAUP=0.D0
	VXGGAUP=0.D0
	EXGGADN=0.D0
	VXGGADN=0.D0
******
	IF(IGGA.EQ.1)     !!! GGA91 Exchange for spin up
     &CALL GGA91EXCH(2.D0*ROUP,GRUP2,G2UP2,2.D0*ALUP,
     &EXLDAUP,VXLDAUP,EXGGAUP,VXGGAUP,
     &DVX2UP,DVXLUP)
	IF(IGGA.EQ.1)     !!! GGA91 Exchange for spin dn
     &CALL GGA91EXCH(2.D0*RODN,GRDN2,G2DN2,2.D0*ALDN,
     &EXLDADN,VXLDADN,EXGGADN,VXGGADN,
     &DVX2DN,DVXLDN)
	IF(IGGA.EQ.2)     !!! GGA96 Exchange for spin up
     &CALL GGA96EXCH(2.D0*ROUP,GRUP2,G2UP2,2.D0*ALUP,
     &EXLDAUP,VXLDAUP,EXGGAUP,VXGGAUP,
     &DVX2UP,DVXLUP)
	IF(IGGA.EQ.2)     !!! GGA96 Exchange for spin dn
     &CALL GGA96EXCH(2.D0*RODN,GRDN2,G2DN2,2.D0*ALDN,
     &EXLDADN,VXLDADN,EXGGADN,VXGGADN,
     &DVX2DN,DVXLDN)
******
 	EXCUP=EXCUP+EXGGAUP
 	EXCDN=EXCDN+EXGGADN
 	VXCUP=VXCUP+VXGGAUP
 	VXCDN=VXCDN+VXGGADN
C  Non-local correlation:
	ECGGA=0.D0
	VCGGAUP=0.D0
	VCGGADN=0.D0
******
	IF(IGGA.EQ.1.AND.ILDA.NE.6) !!! GGA91 Correlation
     &CALL GGA91CORR(ROUP,RODN,GRUP,GRDN,G2UP,G2DN,ALUP,ALDN,
     &ECLDA,VCLDAUP,VCLDADN,ECGGA,VCGGAUP,VCGGADN,
     &DVC2UP,DVC2DN,DVCLUP,DVCLDN)
	IF(IGGA.EQ.2.AND.ILDA.NE.6) !!! GGA96 Correlation
     &CALL GGA96CORR(ROUP,RODN,GRUP,GRDN,G2UP,G2DN,ALUP,ALDN,
     &ECLDA,VCLDAUP,VCLDADN,ECGGA,VCGGAUP,VCGGADN,
     &DVC2UP,DVC2DN,DVCLUP,DVCLDN)
******
 	EXCUP=EXCUP+ECGGA
 	EXCDN=EXCDN+ECGGA
 	VXCUP=VXCUP+VCGGAUP
 	VXCDN=VXCDN+VCGGADN
	END





      SUBROUTINE DEXCVXC(ROUP,RODN,GRUP,GRDN,G2UP,G2DN,ALUP,
     &ALDN,IXC,IVAR,IKEY,NSPIN,DEXCUP,DEXCDN,DVXCUP,DVXCDN)
*************************************************************
*  Calculates variation of the exchange-correlation energy  *
*  and potential with respect to:                           *
*    Ivar:                                                  *
*      1    Rho_up                                          *
*      2    Rho_dn        for NSPIN=2                       *
*    Ivar:                                                  *
*      1    Rho_up+Rho_dn                                   *
*      2    Rho_up+Rho_dn for NSPIN=1                       *
*************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	COMPLEX*16 GRUP(-1:1),GRDN(-1:1),GXUP(-1:1),GXDN(-1:1)
     &    ,G2UP(-1:1,-1:1),G2DN(-1:1,-1:1)
	DIMENSION EXCUP(-2:2),EXCDN(-2:2),VXCUP(-2:2),VXCDN(-2:2)
      DATA PI/3.141592653589793D0/,DVAR/0.001D0/,EPSD/1.D-07/
      IF(ROUP.LT.0D0)ROUP=1.D-10
      IF(RODN.LT.0D0)RODN=1.D-10
C Spin-up density varies
	IF(IVAR.EQ.1.AND.NSPIN.EQ.2)THEN
	DELTA=ROUP*DVAR
	IF(DELTA.LT.EPSD)DELTA=EPSD
	DO ID=-2,2
	RXUP=ROUP+DELTA*DBLE(ID)
      CALL EXCVXC(RXUP,RODN,GRUP,GRDN,G2UP,G2DN,ALUP,ALDN,
     &IXC,IKEY,NSPIN,EXCUP(ID),EXCDN(ID),VXCUP(ID),VXCDN(ID))
	ENDDO
C Spin-dn density varies
	ELSEIF(IVAR.EQ.2.AND.NSPIN.EQ.2)THEN
	DELTA=RODN*DVAR
	IF(DELTA.LT.EPSD)DELTA=EPSD
	DO ID=-2,2
	RXDN=RODN+DELTA*DBLE(ID)
      CALL EXCVXC(ROUP,RXDN,GRUP,GRDN,G2UP,G2DN,ALUP,ALDN,
     &IXC,IKEY,NSPIN,EXCUP(ID),EXCDN(ID),VXCUP(ID),VXCDN(ID))
	ENDDO
C Total density varies
	ELSEIF((IVAR.EQ.1.OR.IVAR.EQ.2).AND.NSPIN.EQ.1)THEN
	DELTA=0.5D0*(ROUP+RODN)*DVAR
	IF(DELTA.LT.EPSD)DELTA=EPSD
	DO ID=-2,2
	RXUP=ROUP+DELTA*DBLE(ID)
	RXDN=RODN+DELTA*DBLE(ID)
      CALL EXCVXC(RXUP,RXDN,GRUP,GRDN,G2UP,G2DN,ALUP,ALDN,
     &IXC,IKEY,NSPIN,EXCUP(ID),EXCDN(ID),VXCUP(ID),VXCDN(ID))
	ENDDO
	ENDIF
C Variations done, set-up derivatives
	DEXCUP=(EXCUP(-2)-EXCUP(2)+8.D0*(EXCUP(1)-EXCUP(-1)))/
     &(24.D0*DELTA/DBLE(NSPIN))
	DEXCDN=(EXCDN(-2)-EXCDN(2)+8.D0*(EXCDN(1)-EXCDN(-1)))/
     &(24.D0*DELTA/DBLE(NSPIN))
	DVXCUP=(VXCUP(-2)-VXCUP(2)+8.D0*(VXCUP(1)-VXCUP(-1)))/
     &(24.D0*DELTA/DBLE(NSPIN))
	DVXCDN=(VXCDN(-2)-VXCDN(2)+8.D0*(VXCDN(1)-VXCDN(-1)))/
     &(24.D0*DELTA/DBLE(NSPIN))
C For exchange-correlation energy in LDA one can use expressions:
C  	IF(IVAR.EQ.1.AND.IGGA.EQ.0)
C    &DEXCUP=(VXCUP(0)-EXCUP(0))/(ROUP+RODN)
C  	IF(IVAR.EQ.2.AND.IGGA.EQ.0)
C    &DEXCDN=(VXCDN(0)-EXCDN(0))/(ROUP+RODN)
	END



      SUBROUTINE GEXCVXC(ROUP,RODN,GRUP,GRDN,G2UP,G2DN,ALUP,
     &ALDN,IXC,IVAR,IKEY,NSPIN,GEXCUP,GEXCDN,GVXCUP,GVXCDN)
*************************************************************
*  Calculates variation of the exchange-correlation energy  *
*  and potential with respect to:                           *
*    Ivar:  for NSPIN=2                                     *
*      1    Nabla_a(Rho_up)             (a=x,y,z/-1,0,1)    *
*      2    Nabla_a(Rho_dn)                                 *
*    Ivar:  for NSPIN=1                                     *
*      1    Nabla_a(Rho_tot)                                *
*      2    Nabla_a(Rho_tot)                                *
*************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	COMPLEX*16 DELTA(-1:1) 
     &    ,GRUP(-1:1),GXUP(-1:1),G2UP(-1:1,-1:1)
     &    ,GRDN(-1:1),GXDN(-1:1),G2DN(-1:1,-1:1)
     &    ,GEXCUP(-1:1),GVXCUP(-1:1)
     &    ,GEXCDN(-1:1),GVXCDN(-1:1)
	DIMENSION 
     &     EXCUP(-2:2,-1:1),EXCDN(-2:2,-1:1)
     &    ,VXCUP(-2:2,-1:1),VXCDN(-2:2,-1:1)
      DATA PI/3.141592653589793D0/,DVAR/0.001D0/,EPSD/1.D-07/
      IF(ROUP.LT.0D0)ROUP=1.D-10
      IF(RODN.LT.0D0)RODN=1.D-10
C Spin-up gradient varies
	IF(IVAR.EQ.1.AND.NSPIN.EQ.2)THEN
	DO MY=-1,1
	DELTA(MY)=GRUP(MY)*DVAR
	IF(ABS(DELTA(MY)).LT.EPSD)DELTA(MY)=EPSD
	DO ID=-2,2
	DO NY=-1,1
	IF(MY.EQ.NY)GXUP(MY)=GRUP(MY)+DELTA(MY)*DBLE(ID)
	IF(MY.NE.NY)GXUP(NY)=GRUP(NY)
	ENDDO
        IF(ID.NE.0)
     &  CALL EXCVXC(ROUP,RODN,GXUP,GRDN,G2UP,G2DN,ALUP,ALDN,
     &  IXC,IKEY,NSPIN,EXCUP(ID,MY),EXCDN(ID,MY),VXCUP(ID,MY),
     &  VXCDN(ID,MY))
	ENDDO
	ENDDO
C Spin-dn gradient varies
	ELSEIF(IVAR.EQ.2.AND.NSPIN.EQ.2)THEN
	DO MY=-1,1
	DELTA(MY)=GRDN(MY)*DVAR
	IF(ABS(DELTA(MY)).LT.EPSD)DELTA(MY)=EPSD
	DO ID=-2,2
	DO NY=-1,1
	IF(MY.EQ.NY)GXDN(MY)=GRDN(MY)+DELTA(MY)*DBLE(ID)
	IF(MY.NE.NY)GXDN(NY)=GRDN(NY)
	ENDDO
      IF(ID.NE.0)
     &CALL EXCVXC(ROUP,RODN,GRUP,GXDN,G2UP,G2DN,ALUP,ALDN,
     &IXC,IKEY,NSPIN,EXCUP(ID,MY),EXCDN(ID,MY),VXCUP(ID,MY),
     &VXCDN(ID,MY))
	ENDDO
	ENDDO
C Total gradient varies
	ELSEIF((IVAR.EQ.1.OR.IVAR.EQ.2).AND.NSPIN.EQ.1)THEN
	DO MY=-1,1
	DELTA(MY)=0.5D0*(GRUP(MY)+GRDN(MY))*DVAR
	IF(ABS(DELTA(MY)).LT.EPSD)DELTA(MY)=EPSD
	DO ID=-2,2
	DO NY=-1,1
	IF(MY.EQ.NY)GXUP(MY)=GRUP(MY)+DELTA(MY)*DBLE(ID)
	IF(MY.EQ.NY)GXDN(MY)=GRDN(MY)+DELTA(MY)*DBLE(ID)
	IF(MY.NE.NY)GXUP(NY)=GRUP(NY)
	IF(MY.NE.NY)GXDN(NY)=GRDN(NY)
	ENDDO
      IF(ID.NE.0)
     &CALL EXCVXC(ROUP,RODN,GXUP,GXDN,G2UP,G2DN,ALUP,ALDN,
     &IXC,IKEY,NSPIN,EXCUP(ID,MY),EXCDN(ID,MY),VXCUP(ID,MY),
     &VXCDN(ID,MY))
	ENDDO
	ENDDO
	ENDIF
C Variations done, set-up derivatives with respect to gradients
	DO MY=-1,1
	GEXCUP(MY)=(EXCUP(-2,MY)-EXCUP(2,MY)+8.D0*(EXCUP(1,MY)-
     &EXCUP(-1,MY)))/(24.D0*DELTA(MY)/DBLE(NSPIN))
	GEXCDN(MY)=(EXCDN(-2,MY)-EXCDN(2,MY)+8.D0*(EXCDN(1,MY)-
     &EXCDN(-1,MY)))/(24.D0*DELTA(MY)/DBLE(NSPIN))
	GVXCUP(MY)=(VXCUP(-2,MY)-VXCUP(2,MY)+8.D0*(VXCUP(1,MY)-
     &VXCUP(-1,MY)))/(24.D0*DELTA(MY)/DBLE(NSPIN))
	GVXCDN(MY)=(VXCDN(-2,MY)-VXCDN(2,MY)+8.D0*(VXCDN(1,MY)-
     &VXCDN(-1,MY)))/(24.D0*DELTA(MY)/DBLE(NSPIN))
	ENDDO
	END



       SUBROUTINE G2VXC(ROUP,RODN,GRUP,GRDN,G2UP,G2DN,ALUP,ALDN,
     &IXC,IVAR,IKEY,NSPIN,DVXC2UP,DVXC2DN,DVXCLUP,DVXCLDN)
*****************************************************************
*  Calculates variation of the exchange-correlation             *
*  potential with respect to:                                   *
*    Ivar   for NSPIN=2                                         *
*     1     Nabla_a*Nabla_b(Rho_up)    (a,b=x,y,z/-1,0,1)       *
*           Nabla^2(Rho_up)            (Laplasian)              *
*     2     Nabla_a*Nabla_b(Rho_dn)                             *
*           Nabla^2(Rho_dn)                                     *
*    Ivar   for NSPIN=1                                         *
*     1     Nabla_a*Nabla_b(Rho_tot)   (a,b=x,y,z/-1,0,1)       *
*           Nabla^2(Rho_tot)           (Laplasian)              *
*     2     Nabla_a*Nabla_b(Rho_tot)                            *
*           Nabla^2(Rho_tot)                                    *
*****************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	COMPLEX*16 
     &    GRUP(-1:1),GRUP2(-1:1),G2UP(-1:1,-1:1),G2UP2(-1:1,-1:1)
     &   ,GRDN(-1:1),GRDN2(-1:1),G2DN(-1:1,-1:1),G2DN2(-1:1,-1:1)
     &   ,DVX2UP(-1:1,-1:1),DVC2UP(-1:1,-1:1),DVXC2UP(-1:1,-1:1)
     &   ,DVX2DN(-1:1,-1:1),DVC2DN(-1:1,-1:1),DVXC2DN(-1:1,-1:1)
        DATA PI/3.141592653589793D0/
C  Set keys for LDA and GGA
	ILDA=MOD(IXC,10)     !!! LDA key
	IGGA=(IXC-ILDA)/10   !!! GGA key
	IF(ILDA.LT.0.OR.ILDA.GT.6)CALL FULLSTOP('<G2VXC>:WRONG ILDA')
	IF(IGGA.LT.0.OR.IGGA.GT.2)CALL FULLSTOP('<G2VXC>:WRONG IGGA')
	DO MY1=-1,1
	DO MY =-1,1
	DVXC2UP(MY1,MY)=(0.D0,0.D0)
	DVXC2DN(MY1,MY)=(0.D0,0.D0)
	ENDDO
	ENDDO
	DVXCLUP=0.D0
	DVXCLDN=0.D0
	IF(ILDA.EQ.0)RETURN
C  ********************************
C  LOCAL SPIN DENSITY APPROXIMATION
C  ********************************
C  Local exchange
      EXLDAUP=0.D0
      VXLDAUP=0.D0
      EXLDADN=0.D0
      VXLDADN=0.D0
******
      CALL LSDEXCH(2.D0*ROUP,EXLDAUP,VXLDAUP)
      CALL LSDEXCH(2.D0*RODN,EXLDADN,VXLDADN)
******
C  Local correlation:
      ECLDA=0.D0
      VCLDAUP=0.D0
      VCLDADN=0.D0
******
      IF(ILDA.EQ.1.OR.ILDA.EQ.2.OR.ILDA.EQ.3)
     &CALL LSDCORB(ROUP,RODN,ECLDA,VCLDAUP,VCLDADN,ILDA)
      IF(ILDA.EQ.4)
     &CALL LSDCORV(ROUP,RODN,ECLDA,VCLDAUP,VCLDADN)
      IF(ILDA.EQ.5)
     &CALL LSDCORP(ROUP,RODN,ECLDA,VCLDAUP,VCLDADN)
******
	IF(IGGA.EQ.0)RETURN
C  *****************************************************
C  GENERALIZED GRADIENT APPROXIMATION AFTER PERDEW ET.AL
C  *****************************************************
C  Non-local exchange:
	DO MY=-1,1
	GRUP2(MY)=2.D0*GRUP(MY)
	GRDN2(MY)=2.D0*GRDN(MY)
	DO MY1=-1,1
	G2UP2(MY1,MY)=2.D0*G2UP(MY1,MY)
	G2DN2(MY1,MY)=2.D0*G2DN(MY1,MY)
	ENDDO
	ENDDO
	DO MY1=-1,1
	DO MY =-1,1
	DVX2UP(MY1,MY)=(0.D0,0.D0)
	DVX2DN(MY1,MY)=(0.D0,0.D0)
	ENDDO
	ENDDO
	DVXLUP=0.D0
	DVXLDN=0.D0
******
	IF(IGGA.EQ.1.AND.IVAR.EQ.1) ! GGA91 Exchange for spin up
     &CALL GGA91EXCH(2.D0*ROUP,GRUP2,G2UP2,2.D0*ALUP,
     &EXLDAUP,VXLDAUP,EXGGAUP,VXGGAUP,DVX2UP,DVXLUP)
	IF(IGGA.EQ.1.AND.IVAR.EQ.2) ! GGA91 Exchange for spin dn
     &CALL GGA91EXCH(2.D0*RODN,GRDN2,G2DN2,2.D0*ALDN,
     &EXLDADN,VXLDADN,EXGGADN,VXGGADN,DVX2DN,DVXLDN)
	IF(IGGA.EQ.2.AND.IVAR.EQ.1) ! GGA96 Exchange for spin up
     &CALL GGA96EXCH(2.D0*ROUP,GRUP2,G2UP2,2.D0*ALUP,
     &EXLDAUP,VXLDAUP,EXGGAUP,VXGGAUP,DVX2UP,DVXLUP)
	IF(IGGA.EQ.2.AND.IVAR.EQ.2) ! GGA96 Exchange for spin dn
     &CALL GGA96EXCH(2.D0*RODN,GRDN2,G2DN2,2.D0*ALDN,
     &EXLDADN,VXLDADN,EXGGADN,VXGGADN,DVX2DN,DVXLDN)
******
	DO MY1=-1,1
	DO MY =-1,1
	DVXC2UP(MY1,MY)=DVXC2UP(MY1,MY)+DVX2UP(MY1,MY)*DBLE(NSPIN)
	DVXC2DN(MY1,MY)=DVXC2DN(MY1,MY)+DVX2DN(MY1,MY)*DBLE(NSPIN)
	ENDDO
	ENDDO
	DVXCLUP=DVXCLUP+DVXLUP
	DVXCLDN=DVXCLDN+DVXLDN
C  Non-local correlation:
	DO MY1=-1,1
	DO MY =-1,1
	DVC2UP(MY1,MY)=(0.D0,0.D0)
	DVC2DN(MY1,MY)=(0.D0,0.D0)
	ENDDO
	ENDDO
	DVCLUP=0.D0
	DVCLDN=0.D0
******
	IF(IGGA.EQ.1.AND.ILDA.NE.6) !!! GGA91 Correlation
     &CALL GGA91CORR(ROUP,RODN,GRUP,GRDN,G2UP,G2DN,ALUP,ALDN,
     &ECLDA,VCLDAUP,VCLDADN,ECGGA,VCGGAUP,VCGGADN,
     &DVC2UP,DVC2DN,DVCLUP,DVCLDN)
	IF(IGGA.EQ.2.AND.ILDA.NE.6) !!! GGA96 Correlation
     &CALL GGA96CORR(ROUP,RODN,GRUP,GRDN,G2UP,G2DN,ALUP,ALDN,
     &ECLDA,VCLDAUP,VCLDADN,ECGGA,VCGGAUP,VCGGADN,
     &DVC2UP,DVC2DN,DVCLUP,DVCLDN)
******
	DO MY1=-1,1
	DO MY =-1,1
	DVXC2UP(MY1,MY)=DVXC2UP(MY1,MY)+DVC2UP(MY1,MY)
	DVXC2DN(MY1,MY)=DVXC2DN(MY1,MY)+DVC2DN(MY1,MY)
	ENDDO
	ENDDO
	DVXCLUP=DVXCLUP+DVCLUP
	DVXCLDN=DVXCLDN+DVCLDN
	END
