	SUBROUTINE RENROV
****************************************************************
*  Renormalize valence density according to the input charge   *
****************************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_CONTROL ,ONLY: LMTO,IASA,NOVR,NPFR              !!! from readini
      USE COM_PARAM   ,ONLY: NATOM,NSORT,NSPIN,NORBS          !!! from readini
     &                      ,IS,ILSV,NHSORT                   !!! from readini
      USE COM_SPHERE  ,ONLY: SMT,SASA,ZNUC,ZCOR               !!! from readini
      USE COM_LATPAR  ,ONLY: PAR,OMEGAC                       !!! from readini
      USE COM_SYMMLM  ,ONLY: SYM,LMSYM,NSYM                   !!! from symlmm
      USE COM_POTDAT  ,ONLY: POT                              !!! from potfull
      USE COM_DENDAT  ,ONLY: RHO,RHOPS,RHOCOR                 !!! from rhofull
      USE COM_POTMTS  ,ONLY: VMT0,VMT                         !!! from potzero
      USE COM_PLWMESH ,ONLY: NPLW,GPLW                        !!! from plwmesh
      USE COM_RADMESH ,ONLY: NRAD,H,R,DR,WEIGHT,RL            !!! from radmesh
      USE COM_LABELS  ,ONLY: TXTAT,TXTEL,TXTSP                !!! from readini
      USE COM_UNIT    ,ONLY: IUN,IUR,IUT                      !!! from setup
C SET LOCAL VARIABLES
	IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8, ALLOCATABLE :: WORK1(:)
      REAL*8, ALLOCATABLE :: WORK2(:)
	COMPLEX*16 DIM,ROVAL,GINT
	DATA PI/3.141592653589793D0/,DIM/(0.D0,1.D0)/
      CALL ABORTNOW()
C MEMORY ALLOCATIONS
      ALLOCATE(WORK1(0:NRADMEM))
      ALLOCATE(WORK2(0:NRADMEM))
	SQPI4=2.D0*SQRT(PI)
	ZVMTS=0.D0
	ZVALT=0.D0
	DO ISPIN=1,NSPIN                      !!! over spins
	DO IATOM=1,NATOM                      !!! over atoms
	ISORT=IS(IATOM)
C  FIND CHARGES WITHIN MTS
	DO IRAD=0,NRAD(ISORT)
	WORK1(IRAD)=RHO(IRAD,1,IATOM,ISPIN,1)*WEIGHT(IRAD,ISORT)/
     &SQRT(4.D0*PI)
	ENDDO
	CALL QD(H(ISORT),WORK1,WORK2,NRAD(ISORT)+1)
	ZVMTS=ZVMTS+WORK2(NRAD(ISORT))
	ZVALT=ZVALT+(ZNUC(ISORT)-ZCOR(ISORT))/DBLE(NSPIN)
      ENDDO                                 !!! over atoms
      ENDDO                                 !!! over spins
C FIND PSEUDOCHARGE
	ZVINT=0.D0
	IF(IASA.EQ.2)THEN
	DO ISPIN=1,NSPIN                      !!! over spins
	DO IPLW=0,NPLW
	CALL MAKEPLW_SINT(GPLW(1,IPLW),GPLW(2,IPLW),GPLW(3,IPLW),GINT)
	ZVINT=ZVINT+RHOPS(IPLW,ISPIN,1)*GINT
	ENDDO
	ENDDO                                 !!! over spins
	ELSEIF(IASA.EQ.1)THEN
      DO ISPIN=1,NSPIN
      DO IATOM=1,NATOM
      ISORT=IS(IATOM)
      ZVINT=ZVINT+RHO(NRAD(ISORT),1,IATOM,ISPIN,1)/SQRT(4.D0*PI)*
     &4.D0*PI/3.D0*(SASA(ISORT)**3-SMT(ISORT)**3) 
      ENDDO
      ENDDO
      ENDIF
	ZVTOT=ZVMTS+ZVINT
	IF(ABS(ZVTOT).LE.1.D-10)
     &CALL FULLSTOP('<RENROV>: INTERNAL ERROR, VALENCE CHARGE IS ZERO')
C  RENORMALIZE VALENCE DENSITY
	ANORM=ZVALT/ZVTOT
	DO IORBS=1,NORBS                  !!! over iorbs
	DO IUPDN=1,NSPIN                  !!! over spins
	DO IPLW=0,NPLW
	ROVAL=RHOPS(IPLW,IUPDN,IORBS)*ANORM
	RHOPS(IPLW,IUPDN,IORBS)=ROVAL
	ENDDO
	DO IATOM=1,NATOM                  !!! over atoms
	ISORT=IS(IATOM)
	DO L=0,ILSV(ISORT)                !!! over l
	DO M=-L,L                         !!! over m
	LM=L*(L+1)+M+1
	IF(SYM(LM,IATOM))THEN
	ISYM=LMSYM(LM,IATOM)
	DO IRAD=0,NRAD(ISORT)
	ROVAL=RHO(IRAD,ISYM,IATOM,IUPDN,IORBS)*ANORM
	RHO(IRAD,ISYM,IATOM,IUPDN,IORBS)=ROVAL
	ENDDO
	ENDIF
	ENDDO                             !!! over m
	ENDDO                             !!! over l
	ENDDO                             !!! over atoms
	ENDDO                             !!! over spins
	ENDDO                             !!! over iorbs
	WRITE(IUN,1010)ZVALT
	WRITE(IUN,1020)ZVTOT
	WRITE(IUN,1030)ANORM
C MEMORY DEALLOCATIONS
      DEALLOCATE(WORK1)
      DEALLOCATE(WORK2)
1010  FORMAT(1X,'Valence charge in whole  elementary cell  must be ',
     &G14.7)
1020  FORMAT(1X,'Valence charge  found  via fourier  transform  is ',
     &G14.7)
1030  FORMAT(1X,'Renormalization coefficient of the val.density is ',
     &G14.7)
	END
