	SUBROUTINE VECGEN
***********************************************************
*  Generate vectors in direct and reciprocal space.       *
***********************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                          !!! from setup
      USE COM_PARAM,   ONLY : NATOM,NSORT,IS,ILST,ILSB,ILSV   !!! from readini
      USE COM_LATPAR,  ONLY : PAR                             !!! from readini
      USE COM_KAPPAR,  ONLY : NPAN,NKAP,EKAP,EK,EDGE          !!! from readini
      USE COM_LATTICE, ONLY : RBAS,GBAS,TAU                   !!! from readstr
      USE COM_KMESH,   ONLY : NPNT,PNT                        !!! from makettr
      USE COM_VECGEN,  ONLY : LIMDT,NIVEC,NRVEC,NGVEC,NSVEC   !!! from vecgen
     &                       ,RVEC,GVEC,SVEC,DELTAU,ALPHEVL
     &                       ,ETAEVLD
      USE COM_LMSTRS,  ONLY : NR2,NR4,ILS2,ILS4,ILD2,ILD4     !!! from vecgen
     &                       ,LIMLPL2,LIMLPL4,LIMLDL2,LIMLDL4
     &                       ,IDELP
      USE COM_SITEGEN, ONLY : NSITES,SITES,IREFSITES          !!! from sitegen
      USE COM_UNIT,    ONLY : IUN,IUR,IUT                     !!! from setup
      USE COM_MASHDEP, ONLY : NVAL,LENINT,LENREL,LENCMP       !!! from setup
C SET LOCAL VARIABLES
	IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER, INTRINSIC :: SIZE
	DIMENSION GL(3),TAU00(3)
	DATA PI/3.141592653589793D0/,TAU00/0.D0,0.D0,0.D0/
      CALL ABORTNOW()
C FIND CONNECTING VECTORS
	ALLOCATE(DELTAU(3,NATOMMEM*NATOMMEM))
	LIMDT=0
	DO IATOM=1,NATOM
	DO JATOM=1,NATOM
	GL(1)=TAU(1,IATOM)-TAU(1,JATOM)
	GL(2)=TAU(2,IATOM)-TAU(2,JATOM)
	GL(3)=TAU(3,IATOM)-TAU(3,JATOM)
	IFND=0
	DO IDT=1,LIMDT
	IF(ABS(GL(1)-DELTAU(1,IDT)).LE.1.D-10.AND
     &  .ABS(GL(2)-DELTAU(2,IDT)).LE.1.D-10.AND
     &  .ABS(GL(3)-DELTAU(3,IDT)).LE.1.D-10)IFND=1
	IF(IFND.EQ.1)EXIT
	ENDDO
	IF(IFND.EQ.0)THEN
	LIMDT=LIMDT+1
	DELTAU(1,LIMDT)=GL(1)
	DELTAU(2,LIMDT)=GL(2)
	DELTAU(3,LIMDT)=GL(3)
	ENDIF
	ENDDO
	ENDDO
	LIMDTMEM=LIMDT
	DEALLOCATE(DELTAU)
C MEMORY ALLOCATIONS FOR MAIN CONTROL ARRAYS
	ALLOCATE(ILS2(LIMDTMEM))
	ALLOCATE(ILS4(LIMDTMEM))
	ALLOCATE(ILD2(LIMDTMEM))
	ALLOCATE(ILD4(LIMDTMEM))
	ALLOCATE(LIMLPL2(LIMDTMEM))
	ALLOCATE(LIMLPL4(LIMDTMEM))
	ALLOCATE(LIMLDL2(LIMDTMEM))
	ALLOCATE(LIMLDL4(LIMDTMEM))
	ALLOCATE(IDELP(NATOMMEM,NATOMMEM))
	ALLOCATE(DELTAU(3,LIMDTMEM))
C INPUT # = NIVEC, SET UP ACCURACY, EPS=EXP(-ACCU*ACCU) 
	ACCU=(3.D0*SQRT(PI)/4.D0*DBLE(NIVEC))**(1.D0/3.D0)
C SET UP # OF VECTORS FROM THE CONDITION OF FASTEST CALCULATION
C WITH THE FIXED ACCURACY "ACCU". TIME UNITS=SPH.HARM.CALCULATION
C     TIMED=DBLE(NATOM)**2*(NKAP(1)+3.D0*NKAP(1)+1.D0)
C     TIMER=DBLE(NATOM)**2*(NKAP(1)+1.D0/DBLE(NATOM))
	XN=4.D0    !!! UNIVERSAL; XN=TIMED/TIMER, SEE ABOVE FORMULAS 
	IF(NATOM.EQ.1.AND.NKAP(1).EQ.1)XN=2.5D0 ! SEE ABOVE FORMULAS
	XN=XN*ALPHEVL
	NG=AINT(DBLE(NIVEC)*SQRT(XN))
	NR=AINT(DBLE(NIVEC)/SQRT(XN))
C DEFINE SUMMATION SPHERES
	CALL CROSS(GL,RBAS(1,2),RBAS(1,3))
	OMEGAR=ABS(RBAS(1,1)*GL(1)+RBAS(2,1)*GL(2)+RBAS(3,1)*GL(3)) 
	RMAXR=(3.D0*DBLE(NR)*OMEGAR/4.D0/PI)**(1.D0/3.D0)+1.D-04
	CALL CROSS(GL,GBAS(1,2),GBAS(1,3))
	OMEGAG=ABS(GBAS(1,1)*GL(1)+GBAS(2,1)*GL(2)+GBAS(3,1)*GL(3)) 
	RMAXG=(3.D0*DBLE(NG)*OMEGAG/4.D0/PI)**(1.D0/3.D0)+1.D-04
C GENERATE DIRECT LATTICE VECTORS
	CALL SITEGEN(RMAXR,RBAS,TAU00,1)
	NRVEC=NSITES+1
	ALLOCATE(RVEC(3,NRVEC))
	SIZE_RVEC=DBLE(SIZE(RVEC)*LENREL)/1024.D0**2
	IVEC=0
	DO ISITES=0,NSITES
	IVEC=IVEC+1
	RVEC(1,IVEC)=SITES(1,ISITES)
	RVEC(2,IVEC)=SITES(2,ISITES)
	RVEC(3,IVEC)=SITES(3,ISITES)
	ENDDO
	DEALLOCATE(SITES)
	DEALLOCATE(IREFSITES)
C GENERATE RECIPROCAL LATTICE VECTORS
	CALL SITEGEN(RMAXG,GBAS,TAU00,1)
	NGVEC=NSITES+1
	ALLOCATE(GVEC(3,NGVEC))
	SIZE_GVEC=DBLE(SIZE(GVEC)*LENREL)/1024.D0**2
	IVEC=0
	DO ISITES=0,NSITES
	IVEC=IVEC+1
	GVEC(1,IVEC)=SITES(1,ISITES)
	GVEC(2,IVEC)=SITES(2,ISITES)
	GVEC(3,IVEC)=SITES(3,ISITES)
	ENDDO
	DEALLOCATE(SITES)
	DEALLOCATE(IREFSITES)
C SET UP EVALD SPLITTING <ETAEVLD>
	ETA0=SQRT(PI)/OMEGAR**(1.D0/3.D0)
	ETAEVLD=ETA0*XN**(1.D0/6.D0)
C SET UP # OF VECTORS FOR DIRECT SUMMATION
	RMAXS=(3.D0*DBLE(NR+NG)*OMEGAR/4.D0/PI)**(1.D0/3.D0)+1.D-04
	CALL SITEGEN(RMAXS,RBAS,TAU00,1)
	NSVEC=NSITES+1
	ALLOCATE(SVEC(3,NSVEC))
	SIZE_SVEC=DBLE(SIZE(SVEC)*LENREL)/1024.D0**2
	IVEC=0
	DO ISITES=0,NSITES
	IVEC=IVEC+1
	SVEC(1,IVEC)=SITES(1,ISITES)
	SVEC(2,IVEC)=SITES(2,ISITES)
	SVEC(3,IVEC)=SITES(3,ISITES)
	ENDDO
	DEALLOCATE(SITES)
	DEALLOCATE(IREFSITES)
	EDGE=-ACCU**4/RMAXS**2/PAR**2
	CURMEM=CURMEM+SIZE_RVEC+SIZE_GVEC+SIZE_SVEC
C FIND CONNECTING VECTORS
	LIMDT=0
	DELMAX=0.D0
	DO IATOM=1,NATOM
	DO JATOM=1,NATOM
	GL(1)=TAU(1,IATOM)-TAU(1,JATOM)
	GL(2)=TAU(2,IATOM)-TAU(2,JATOM)
	GL(3)=TAU(3,IATOM)-TAU(3,JATOM)
	IFND=0
	DO IDT=1,LIMDT
	IF(ABS(GL(1)-DELTAU(1,IDT)).LE.1.D-10.AND
     &  .ABS(GL(2)-DELTAU(2,IDT)).LE.1.D-10.AND
     &  .ABS(GL(3)-DELTAU(3,IDT)).LE.1.D-10)IFND=1
	IF(IFND.EQ.1)EXIT
	ENDDO
	IF(IFND.EQ.0)THEN
	LIMDT=LIMDT+1
	DELTAU(1,LIMDT)=GL(1)
	DELTAU(2,LIMDT)=GL(2)
	DELTAU(3,LIMDT)=GL(3)
	CALL REDUC2(GL(1),GL(2),GL(3),NRVEC,RVEC,
     &  DRX,DRY,DRZ,RX,RY,RZ)
	DELMAX=MAX(DELMAX,SQRT(DRX*DRX+DRY*DRY+DRZ*DRZ))
	IDELP(IATOM,JATOM)=LIMDT
	ELSE
	IDELP(IATOM,JATOM)=IDT
	ENDIF
	ENDDO
	ENDDO
C  FIND MAX.LMAX FOR CONNECTING VECTORS
	ILS2MAX=0
	ILS4MAX=0
	DO IDT=1,LIMDT
	ILS2(IDT)=0
	ILD2(IDT)=0
	ILS4(IDT)=0
	ILD4(IDT)=-1
	DO IATOM=1,NATOM
	DO JATOM=1,NATOM
	IF(IDELP(IATOM,JATOM).EQ.IDT)THEN
	LS2=1+ILST(IS(IATOM))+ILSB(IS(JATOM))
	LD2=1+ILSB(IS(IATOM))+ILSB(IS(JATOM))
	LS4=ILSV(IS(IATOM))+ILSV(IS(JATOM))+2
	LD4=-1
	ILS2(IDT)=MAX(ILS2(IDT),LS2)
	ILD2(IDT)=MAX(ILD2(IDT),LD2)
	ILS4(IDT)=MAX(ILS4(IDT),LS4)
	ILD4(IDT)=MAX(ILD4(IDT),LD4)
	ILS2MAX=MAX(ILS2MAX,ILS2(IDT))
	ILS4MAX=MAX(ILS4MAX,ILS4(IDT))
	ENDIF
	ENDDO
	ENDDO
	LIMLPL2(IDT)=(ILS2(IDT)+1)**2
	LIMLDL2(IDT)=(ILD2(IDT)+1)**2
	LIMLPL4(IDT)=(ILS4(IDT)+1)**2
	LIMLDL4(IDT)=(ILD4(IDT)+1)**2
	ENDDO
C  ESTIMATE RECORD LENGHT FOR STR.CON
	LIM2=0
	LIM4=0
	DO ITAU=1,LIMDT
	LIM2=MAX(LIM2,LIMLPL2(ITAU)+LIMLDL2(ITAU))
	LIM4=MAX(LIM4,LIMLPL4(ITAU))
	ENDDO
	NR2=LIM2*NVAL+NVAL/2+NVAL/2
	NR4=LIM4*NVAL+NVAL/2
C  DEFINE MAXIMAL K-VECTOR
	PNTMAX=0.D0
	DO IPAN=1,NPAN      
	DO IPNT=1,NPNT(IPAN)
	CALL REDUC2(PNT(1,IPNT,IPAN),PNT(2,IPNT,IPAN),
     &PNT(3,IPNT,IPAN),NGVEC,GVEC,XK,YK,ZK,GX,GY,GZ)
	PNTMAX=MAX(PNTMAX,SQRT(XK*XK+YK*YK+ZK*ZK))
	ENDDO
	ENDDO
C  PRINT CALCULATED VALUES
	EPSR=EXP(-(RMAXR**2+DELMAX**2-2.D0*RMAXR*DELMAX)
     &  *ETAEVLD*ETAEVLD)
	EPSG=EXP(-(RMAXG**2+PNTMAX**2-2.D0*RMAXG*PNTMAX)
     &  /ETAEVLD/ETAEVLD/4.D0*
     &  (2.D0*PI)**2) 
	EPSS=EXP(-(RMAXS**2+DELMAX**2-2.D0*RMAXS*DELMAX)**0.5D0*
     &  SQRT(-EDGE)*PAR)
	WRITE(IUN,1010)
	WRITE(IUN,1020)RMAXR,EPSR,NRVEC
	WRITE(IUN,1030)RMAXG,EPSG,NGVEC
	WRITE(IUN,1040)RMAXS,EPSS,NSVEC
	WRITE(IUN,1050)EDGE
	WRITE(IUN,1060)LIMDT
1010  FORMAT(/1X,'Result from VECGEN for direct/reciprocal spaces -->')
1020  FORMAT(1X,'Rmax=',G14.7,'; Accuracy=',G14.7,'; # of vectors=',I6)
1030  FORMAT(1X,'Gmax=',G14.7,'; Accuracy=',G14.7,'; # of vectors=',I6)
1040  FORMAT(1X,'Smax=',G14.7,'; Accuracy=',G14.7,'; # of vectors=',I6)
1050  FORMAT(1X,'Min.energy for using Evald"s method=',G14.7,' Ry')
1060  FORMAT(1X,'Total # of connecting vectors found ',I7/)
	END


	SUBROUTINE CROSS(A,B,C)
**********************************************************
*    Cross product (AX,AY,AZ)=(BX,BY,BZ)*(CX,CY,CZ)      *
**********************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	DIMENSION A(3),B(3),C(3)
	A(1)=B(2)*C(3)-B(3)*C(2)
	A(2)=B(3)*C(1)-B(1)*C(3)
	A(3)=B(1)*C(2)-B(2)*C(1)
	RETURN
	END



	SUBROUTINE REDUC2(DX,DY,DZ,NVEC,VEC,DRX,DRY,DRZ,RX,RY,RZ)
*****************************************************************
*  Represents vector D as Dr-R, where Dr has a minimal lenght   *
*  and R is one of the lattice vectors VEC.                     *
*****************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	DIMENSION VEC(3,NVEC)
	DRX=DX
	DRY=DY
	DRZ=DZ
	ABSDR=SQRT(DRX*DRX+DRY*DRY+DRZ*DRZ)
	RX=0.D0
	RY=0.D0
	RZ=0.D0
	DO 10 IVEC=1,NVEC
	TRX=DX+VEC(1,IVEC)
	TRY=DY+VEC(2,IVEC)
	TRZ=DZ+VEC(3,IVEC)
	ABSTR=SQRT(TRX*TRX+TRY*TRY+TRZ*TRZ)
	IF(ABSTR.GE.ABSDR)GOTO10
	DRX=TRX
	DRY=TRY
	DRZ=TRZ
	ABSDR=ABSTR
	RX=VEC(1,IVEC)
	RY=VEC(2,IVEC)
	RZ=VEC(3,IVEC)
  10  CONTINUE
	END



	SUBROUTINE STRSNG
****************************************************************
*    Check the singularities in the structure constants        *
*    by calculating the difference between free-electron       *
*    energies and input tail energies.                         *
****************************************************************
C SET EXTERNAL CONNECTIONS
      USE COM_PARAM,  ONLY : NATOM,NSORT,IS,ILST,ILSB,ILSV    !!! from readini
      USE COM_KAPPAR, ONLY : NPAN,NKAP,EKAP,EK                !!! from readini
      USE COM_KMESH,  ONLY : NPNT,PNT                         !!! from tetra
      USE COM_VECGEN, ONLY : LIMDT,DELTAU,NRVEC,RVEC,NGVEC,GVEC!! from vecgen
      USE COM_UNIT,   ONLY : IUN,IUR,IUT                      !!! from setup
C SET LOCAL VARIABLES
	IMPLICIT REAL*8 (A-H,O-Z)
	DATA PI/3.141592653589793D0/
C START CODE
	DO IPAN=1,1 ! NPAN
	DO IKAP=1,NKAP(IPAN)
	DIFMIN=10.D+10
	DO IPNT=1,NPNT(IPAN)
	DO IGVEC=1,NGVEC
	GKX=GVEC(1,IGVEC)+PNT(1,IPNT,IPAN)
	GKY=GVEC(2,IGVEC)+PNT(2,IPNT,IPAN)
	GKZ=GVEC(3,IGVEC)+PNT(3,IPNT,IPAN)
	GK2=GKX*GKX+GKY*GKY+GKZ*GKZ
	DIF=ABS(GK2-DREAL(EK(IKAP,IPAN))/4.D0/PI/PI)
	DIFMIN=MIN(DIFMIN,DIF)
	ENDDO 
	ENDDO  
	WRITE(IUN,1010)IKAP,DIFMIN
	IF(DIFMIN+ABS(DIMAG(EK(IKAP,IPAN))).LE.1.D-10)THEN
      WRITE(IUT,*)'Warning from <STRMSH>:' 
	WRITE(IUT,*)'Using positive tail emergies without'
	WRITE(IUT,*)'small imaginary part will make structure'
	WRITE(IUT,*)'constants singular.'
	ENDIF
 	ENDDO 
	ENDDO
1010  FORMAT(1X,'Minimum difference between |k+G|^2 and kappa',
     &  I1,'^2 is ',G14.7)
	END
